/**
 * @name change_price_timeout
 * @type {array}
 * @description 处理短时间内多次点击加减按钮
 *              将加减事件添加到定时器
 *              并将定时器压入数组
 *              如果在一定时间（例如：300ms）内没有其他加减事件
 *              则向服务器发送请求
 *              否则替换数组内的定时器
 */
var change_price_timeout = new Array()

/**
 * @function
 * @name checkout
 * @description 跳转到结算页面
 */
function checkout() {
    var selected_goods = get_selected_goods()
    var goods_number = new Object()
    for (i in selected_goods) {
        var goods = $('#number_' + selected_goods[i])
        goods_number[selected_goods[i]] = $(goods).val()
    }
    if (_.isEmpty(selected_goods)) {
        open_toast('请选择商品')
        return false
    }
    open_checkout(selected_goods, goods_number)
}

/**
 * @function
 * @name clear
 * @description 清空购物车
 */
function clear() {
    ajax({
        data : {
            script_name : 'flow',
            step : 'clear',
        },
        success : cb_clear,
        type : 'GET'
    })
}

/**
 * @function
 * @name cb_clear
 * @description 清空购物车回调函数
 * @param {string} data 服务器端返回的数据
 */
function cb_clear(data) {
    var json = parse_result(data)
    if (json.error <= 0) {
        open_toast(json.content)
        reload()
    }
}

/**
 * @function
 * @name increase_goods
 * @description 增加商品数量
 * @param {string} rec_id 购物车中的商品的唯一标识
 */
function increase_goods(rec_id) {
    $('#number_' + rec_id).val(parseInt($('#number_' + rec_id).val()) + 1)
    pre_action(rec_id)
}

/**
 * @function
 * @name reduce_goods
 * @description 减少商品数量
 * @param {string} rec_id 购物车中的商品的唯一标识
 */
function reduce_goods(rec_id) {
    if ($('#number_' + rec_id).val() > 1) {
        $('#number_' + rec_id).val(parseInt($('#number_' + rec_id).val()) - 1)
        pre_action(rec_id)
    }
}

/**
 * @function
 * @name delete_goods
 * @description 删除商品
 * @param {string} rec_id 购物车中的商品的唯一标识
 */
function delete_goods(rec_id) {
    open_confirm({
        message : '是否删除此商品',
        callback : function(index) {
            if (index == 1) {
                ajax({
                    data : {
                        script_name : 'flow',
                        step : 'drop_goods',
                        id : rec_id,
                    },
                    success : cb_delete_goods,
                    type : 'GET',
                })
            }
        }
    })
}

/**
 * @function
 * @name cb_delete_goods
 * @description 删除商品回调函数
 * @param {string} data 服务器端返回的数据
 */
function cb_delete_goods(data) {
    var json = JSON.parse(data)
    if (json.error <= 0) {
        open_toast(json.content)
        reload()
    }
}

/**
 * @function
 * @name pre_action
 * @description 加减商品数量时先判断是否正在加减数量
 *              加减商品数量时先判断是否正在加减数量
 *              将当前操作加入到定时器中
 * @param {string} rec_id 购物车中的商品的唯一标识
 */
function pre_action(rec_id) {
    if ( typeof (change_price_timeout.rec_id) != 'undefined') {
        clearTimeout(change_price_timeout.rec_id);

    }
    change_price_timeout.rec_id = setTimeout(function() {
        change_price(rec_id);
    }, 200)
    return false
}

/**
 * @function
 * @name change_price
 * @description 根据当前属性和数量获取商品价格
 * @param {string} rec_id 购物车中的商品的唯一标识
 */
function change_price(rec_id) {
    var selected_goods_arr = get_selected_goods()
    var number = $('#number_' + rec_id).val()
    var goods_id = $('#number_' + rec_id).attr('goods_id')
    var suppid = $('#number_' + rec_id).attr('supplier_id')
    var is_package = $('#number_' + rec_id).attr('package_buy')
    
    ajax({
        data : {
            script_name : 'flow',
            step : 'update_group_cart',
            sel_goods : selected_goods_arr.join(','),
            rec_id : rec_id,
            number : number,
            goods_id : goods_id,
            suppid : suppid,
            is_package : is_package,
        },
        success : cb_change_price,
        type : 'GET',
    })
}

/**
 * @function
 * @name cb_change_price
 * @description 获取价格回调函数
 * @param {string} data 服务器端返回的数据
 */
function cb_change_price(data) {
    var json = parse_result(data)
    if (json.error > 0) {
        var rec_id = json.rec_id
        var number = json.number
        $('.input_number').each(function() {
            if ($(this).attr('rec_id') == rec_id) {
                $(this).val(number)
            }
        })
    } else {
        var rec_id = json.rec_id;
        var subtotal = json.subtotal;
        var cart_num = json.cart_num
        publish(CHANNEL_UPDATE_CART, cart_num)
        $('#goods_subtotal_' + rec_id).html(subtotal);
        var cart_amount_desc = json.cart_amount_desc;
        $('#cart_amount_desc').html(cart_amount_desc);
    }
}

/**
 * @function
 * @name get_selected_goods
 * @description 获取选择的商品
 * @returns {array} selected_goods_arr 选择的商品
 */
function get_selected_goods() {
    var selected_goods_arr = get_checkbox_value({
        name : 'goods'
    })
    return selected_goods_arr;
}

/**
 * @function
 * @name select_goods
 * @description 选择商品
 */
function select_goods() {
    var selected_goods = get_selected_goods();
    ajax({
        data : {
            script_name : 'flow',
            act : 'selcart',
            sel_goods : selected_goods.join(','),
        },
        success : cb_select_goods,
        type : 'GET',
    })
}

/**
 * @function
 * @name cb_select_goods
 * @description 选择商品回调函数
 * @param {string} data 服务器端返回的数据
 */
function cb_select_goods(data) {
    var json = JSON.parse(data);
    if (json.error) {
        $('#cart_amount_desc').html(json.message).siblings().addClass('uhide')
    } else {
        var total_price = json.total_price;
        $('#cart_amount_desc').html(total_price).siblings().removeClass('uhide')
    }
}

/**
 * @function
 * @name choose_gift
 * @description 获取赠品
 * @param {string} suppid 店铺唯一标识
 */
function choose_gift(suppid) {
    var sel_goods = get_selected_goods()
    ajax({
        data : {
            script_name : 'flow',
            is_ajax : 1,
            suppid : suppid,
            sel_goods : sel_goods.join(','),
        },
        success : cb_choose_gift,
        type : 'GET',
    })
}

/**
 * @function
 * @name cb_choose_gift
 * @description 获取赠品回调函数
 * @param {string} data 服务器端返回的数据
 */
function cb_choose_gift(data) {
    var json = JSON.parse(data)
    $('#goods_gift_box').html(json.result)
    open_popup({
        content : '#goods_gift_box',
        close_button : false,
    })
    init_tab()
    $('._tab').on('click', function() {
        var tab_key = $(this).attr('tab_key')
        var act_id = $(this).attr('act_id')
        change_gift_confirm_button(tab_key)
    })
    init_checkbox()
    $('._checkbox').on('click', function() {
        change_gift_confirm_button()
    })
    $('#confirm_button').on('click', function() {
        tab_key = $('._tab').not(function() {
            return !$(this).hasClass('selected')
        }).attr('tab_key')
        var act_id = $('#tab_' + tab_key).attr('act_id')
        confirm_select_gift(tab_key, act_id)
    })
}

/**
 * @function
 * @name change_gift_confirm_button
 * @description 修改确认赠品按钮的显示状态
 *              如果没有选择任何赠品，则显示为取消
 *              如果选择赠品，则显示为确认
 * @param {string} tab_key 赠品区域唯一标识
 */
function change_gift_confirm_button(tab_key) {
    if ( typeof tab_key == 'undefined') {
        tab_key = $('._tab').not(function() {
            return !$(this).hasClass('selected')
        }).attr('tab_key')
    }
    var gift = get_checkbox_value({
        name : 'gift_' + tab_key
    })
    if (gift.length <= 0) {
        $('#confirm_button').addClass('cancel').html('取消')
    } else {
        $('#confirm_button').removeClass('cancel').html('确定')
    }
}

/**
 * @function
 * @name change_gift_confirm_button
 * @description 选择赠品
 * @param {string} tab_key 赠品区域唯一标识
 * @param {string} act_id 活动唯一标识
 */
function confirm_select_gift(tab_key, act_id) {
    var gift = get_checkbox_value({
        name : 'gift_' + tab_key
    })
    if (gift.length > 0) {
        ajax({
            data : {
                script_name : 'flow',
                step : 'add_favourable',
                act_id : act_id,
                gift : gift,
            },
            success : cb_confirm_select_gift,
        })
    } else {
        close_popup()
    }
}

/**
 * @function
 * @name cb_confirm_select_gift
 * @description 选择赠品回调函数
 * @param {string} data 服务器端返回的数据
 */
function cb_confirm_select_gift(data) {
    close_popup()
    var json = parse_result(data)
    if (json) {
        open_toast(json.content)
        reload()
    }
}