/**
 * @function
 * @name check_consignee
 * @description 检测订单是否填写了收货地址
 * @returns {boolean} 是否填写了收货地址
 */
function check_consignee() {
    if ($('#address').attr('address_id') > 0) {
        return true
    } else {
        return false
    }
}

/**
 * @function
 * @name get_post_script
 * @description 获取订单附言
 * @returns {string} 订单附言
 */
function get_post_script() {
    return $('#post_script').val();
}

/**
 * @function
 * @name get_shipping
 * @description 获取选择的配送方式
 * @returns {object.<supplier_id:string,shipping_id:string>} shipping 配送方式
 */
function get_shipping() {
    var shipping = new Object()
    $('.supplier_box').each(function() {
        var supplier_id = $(this).attr('suppid')
        shipping[supplier_id] = get_checkbox_value({name:'shipping_'+supplier_id})[0] || '0'
    })
    return shipping
}

/**
 * @function
 * @name get_payment_other
 * @description 获取选择的其它支付方式
 * @returns {string} payment_other 支付方式的唯一标识
 */
function get_payment_other() {
    var payment_other = get_checkbox_value({name:'payment_other'})[0] || '0'
    return payment_other
}

/**
 * @function
 * @name get_payment
 * @description 获取选择的支付方式
 * @returns {string} payment 支付方式的唯一标识
 */
function get_payment() {
    var payment = get_checkbox_value({name:'payment'})[0] || '0'
    return payment
}

/**
 * @function
 * @name get_delivery_time
 * @description 获取送货时间
 * @returns {string} delivery_time 送货时间
 */
function get_delivery_time() {
    var delivery_time = get_checkbox_value({name:'delivery_time'})[0] || '仅工作日送货'
    return delivery_time
}

/**
 * @function
 * @name get_bonus
 * @description 获取优惠券
 * @returns {string} bonus 优惠券
 */
function get_bonus() {
    var bonus = new Object()

    $('.bonus').each(function() {
        bonus[$(this).attr('key')] = $(this).val()
    })

    return JSON.stringify(bonus)
}

/**
 * @function
 * @name get_bonus_sn
 * @description 获取手动输入的优惠券
 * @returns {string} bonus_sn 优惠券代码
 */
function get_bonus_sn() {
    var bonus_sn = new Object();

    $('.bonus_sn').each(function() {
        bonus_sn[$("this").attr('key')] = $("this").val();
    })

    return JSON.stringify(bonus_sn)
}

/**
 * @function
 * @name get_surplus
 * @description 获取余额
 * @returns {string} surplus 余额
 */
function get_surplus() {
    return $('#surplus').val()
}

/**
 * @function
 * @name get_need_inv
 * @description 是否需要发票
 * @returns {number} 需要发票，返回1
 *                   其它返回0
 */
function get_need_inv() {
    if ($('#invoice_switch').data('checked')) {
        return 1
    } else {
        return 0
    }
}

/**
 * @function
 * @name get_inv_type
 * @description 获取发票类型
 * @returns {string} inv_type 发票类型
 */
function get_inv_type() {
    return $('#inv_type').val();
}

/**
 * @function
 * @name get_inv_content
 * @description 获取发票内容
 * @returns {string} inv_content 发票内容
 */
function get_inv_content() {
    return $('#inv_content').val();
}

/**
 * @function
 * @name get_inv_payee_type
 * @description 获取发票抬头类型
 * @returns {string} inv_payee_type 发票抬头类型
 */
function get_inv_payee_type() {
    var inv_payee_type = get_checkbox_value({name:'inv_payee_type'})[0] || '0'
    return inv_payee_type
}

/**
 * @function
 * @name get_inv_payee
 * @description 获取发票抬头
 * @returns {string} inv_payee 发票抬头
 */
function get_inv_payee() {
    return $('#inv_payee').val();
}

/**
 * @function
 * @name get_how_oos
 * @description 获取缺货处理
 * @returns {string} how_oos 缺货处理唯一标识
 */
function get_how_oos() {
    var how_oos = get_checkbox_value({name:'how_oos'})[0] || '0'
    return how_oos
}

/**
 * @function
 * @name select_shipping
 * @description 更改配送方式
 * @param {string} shipping_id 配送方式唯一标识
 * @param {string} supplier_id 店铺唯一标识
 */
function select_shipping(shipping_id, supplier_id) {
    ajax({
        data : {
            script_name : 'flow',
            step : 'select_shipping',
            recid : shipping_id,
            suppid : supplier_id,
        },
        success : cb_select_shipping,
        type : 'GET',
    })
}

/**
 * @function
 * @name cb_select_shipping
 * @description 更改配送方式回调函数
 * @param {string} data 服务器端返回的数据
 */
function cb_select_shipping(data) {
    var json = JSON.parse(data)
    var supplier_id = json.suppid
    var pickup_id = json.pickup_id
    $('#order_total').html(json.content)
    $('#order_total_label').html($('#amount_formated').val())
    $('#order_supplier_shipping').html(json.supplier_shipping)
    if ( typeof json.picktxt != 'undefined' && json.picktxt != '') {
        $('#select_pickup_point_' + supplier_id).html(json.picktxt).removeClass('uhide').off('click').on('click', function() {
            select_pickup_info(supplier_id, pickup_id)
        })
    } else {
        $('#select_pickup_point_' + supplier_id).addClass('uhide')
    }
    if ($('#surplus_switch').data('checked')) {
        $('#surplus_switch').trigger('tap')
    }
}

/**
 * @function
 * @name select_pickup_info
 * @description 更改门店自提点
 * @param {string} supplier_id 店铺唯一标识
 * @param {string} pickup_id 自提点唯一标识
 */
function select_pickup_info(supplier_id, pickup_id) {
    ajax({
        data : {
            script_name : 'flow',
            step : 'select_pickinfo',
            sid : supplier_id,
            pid : pickup_id,
        },
        success : cb_select_pickup_info,
    })
}

/**
 * @function
 * @name cb_select_pickup_info
 * @description 更改门店自提点回调函数
 * @param {string} data 服务器端返回的数据
 */
function cb_select_pickup_info(data) {
    var json = parse_result(data)
    if (json.error <= 0) {
        open_popup({
            content : json.content,
            close_button : false,
        })
        init_checkbox()
        init_confirm_pickup()
    }
}

/**
 * @function
 * @name init_confirm_pickup
 * @description 初始化确认自提点按钮
 */
function init_confirm_pickup() {
    $('#confirm_pickup_point').on('click', function() {
        var supplier_id = $(this).attr('supplier_id')
        var pickup_id = get_checkbox_value({name:'pickup_point'})[0]
        save_pickup_info(supplier_id, pickup_id)
        close_popup()
    })
}

/**
 * @function
 * @name save_pickup_info
 * @description 确定自提点
 * @param {string} supplier_id 店铺唯一标识
 * @param {string} pickup_id 自提点唯一标识
 */
function save_pickup_info(supplier_id, pickup_id) {
    ajax({
        data : {
            script_name : 'flow',
            step : 'save_point',
            sid : supplier_id,
            pid : pickup_id,
        },
        success : cb_save_pickup_info,
        type : 'GET',
    })
}

/**
 * @function
 * @name cb_save_pickup_info
 * @description 确定自提点回调函数
 * @param {string} data 服务器端返回的数据
 */
function cb_save_pickup_info(data) {
    var json = parse_result(data)
    if (json.error <= 0) {
        var supplier_id = json.suppid
        var pickup_id = json.pickup_id
        if ( typeof json.picktxt != 'undefined' && json.picktxt != '') {
            $('#select_pickup_point_' + supplier_id).html(json.picktxt).removeClass('uhide').off('click').on('click', function() {
                select_pickup_info(supplier_id, pickup_id)
            })
        } else {
            $('#select_pickup_point').addClass('uhide')
        }
    }
}

/**
 * @function
 * @name select_payment
 * @description 选择支付方式
 * @param {string} pay_id 支付方式唯一标识
 */
function select_payment(pay_id) {
    ajax({
        data : {
            script_name : 'flow',
            step : 'select_payment',
            payment : pay_id,
        },
        success : cb_select_payment,
    })
}

/**
 * @function
 * @name cb_select_payment
 * @description 选择支付方式回调函数
 * @param {string} data 服务器端返回的数据
 */
function cb_select_payment(data) {
    var json = parse_result(data)
    if (json && json.error <= 0) {
        $('#order_total').html(json.content)
        $('#order_total_label').html($('#amount_formated').val())
    }
}

/**
 * @function
 * @name validate_bonus
 * @description 验证输入的优惠券是否合法
 * @param {string} bonus_sn 用户输入的优惠券
 * @param {string} supplier_id 店铺唯一标识
 */
function validate_bonus(bonus_sn, supplier_id) {
    if (bonus_sn == '') {
        open_toast('请输入优惠券代码')
        $('#bonus_sn_' + supplier_id).focus()
    } else {
        ajax({
            data : {
                script_name : 'flow',
                step : 'validate_bonus',
                suppid : supplier_id,
                bonus_sn : bonus_sn,
            },
            success : cb_validate_bonus,
            type : 'GET',
        })
    }
}

/**
 * @function
 * @name cb_validate_bonus
 * @description 验证优惠券回调函数
 * @param {string} data 服务器端返回的数据
 */
function cb_validate_bonus(data) {
    var json = parse_result(data)
    var suppid = json.suppid
    if (json.error <= 0) {
        $('#order_total').html(json.content)
        $('#order_total_label').html($('#amount_formated').val())
    }
}

/**
 * @function
 * @name change_surplus
 * @description 更改余额
 * @param {string} val 用户输入的余额
 */
function change_surplus(val) {
    var surplus = val
    if (parseInt(surplus) <= 0) {
        surplus = 0
    }
    ajax({
        data : {
            script_name : 'flow',
            step : "change_surplus",
            surplus : surplus,
        },
        success : cb_change_surplus,
        type : 'GET',
    })
}

/**
 * @function
 * @name cb_change_surplus
 * @description 更改余额回调函数
 * @param {string} data 服务器端返回的数据
 */
function cb_change_surplus(data) {
    var json = parse_result(data)
    if (json && json.error <= 0) {
        var show = json.show
        var surplus = typeof (json.surplus) == 'undefined' ? 0 : parseFloat(json.surplus)
        var content = json.content
        $('#surplus').val(surplus)

        //余额足够支付全部订单
        if (show) {
            $(get_checkbox_target({name:'payment_item',value:pay_balance_id,checked:false})[0]).trigger('click')
            $('#paymen_container').addClass('uhide')
        }
        //余额不足支付全部订单
        else {
            $('#paymen_container').removeClass('uhide')
            $(get_checkbox_target({name:'payment_item',checked:false})[0]).trigger('click')
        }
        $('#order_total').html(json.content)
        $('#order_total_label').html($('#amount_formated').val())
    }
}

/**
 * @function
 * @name get_pickup_point
 * @description 获取选择的门店自提点
 * @returns {string} pickup_point 门店自提点
 */
function get_pickup_point() {
    var pickup_point = new Object
    $('.pickup_point').each(function() {
        var pickup_id = $(this).attr('pickup_id')
        var supplier_id = $(this).attr('supplier_id')
        pickup_point[supplier_id] = pickup_id
    })
    return JSON.stringify(pickup_point)
}

/**
 * @function
 * @name change_bonus
 * @description 更改选择的优惠券
 * @param {string} bonus 优惠券唯一标识
 * @param {string} supplier_id 店铺唯一标识
 */
function change_bonus(bonus, supplier_id) {
    ajax({
        data : {
            script_name : 'flow',
            step : 'change_bonus',
            bonus : bonus,
            suppid : supplier_id,
        },
        success : cb_change_bonus,
        type : 'GET'
    })
}

/**
 * @function
 * @name cb_change_bonus
 * @description 更改优惠券回调函数
 * @param {string} data 服务器端返回的数据
 */
function cb_change_bonus(data) {
    var json = parse_result(data)
    if (json && json.error <= 0) {
        $('#order_total').html(json.content)
    }
    $('#order_total_label').html($('#amount_formated').val())
    if ($('#surplus_switch').data('checked')) {
        $('#surplus_switch').trigger('tap')
    }
}

/**
 * @function
 * @name change_invoice
 * @description 更改发票
 */
function change_invoice() {
    var need_inv = get_need_inv()
    var inv_type = get_inv_type()
    var inv_content = get_inv_content()
    var inv_payee = get_inv_payee()
    ajax({
        data : {
            script_name : 'flow',
            step : 'change_needinv',
            need_inv : need_inv,
            inv_type : inv_type,
            inv_payee : inv_payee,
            inv_content : inv_content,
        },
        success : cb_change_invoice,
        type : 'GET',
    })
}

/**
 * @function
 * @name cb_change_invoice
 * @description 更改发票回调函数
 * @param {string} 服务器端返回的数据
 */
function cb_change_invoice(data) {
    var json = JSON.parse(data)
    $('#order_total').html(json.content)
    $('#order_total_label').html($('#amount_formated').val())
    if ($('#surplus_switch').data('checked')) {
        $('#surplus_switch').trigger('tap')
    }
}

/**
 * @function
 * @name submit_order
 * @description 提交订单
 */
function submit_order() {
    var have_consignee = check_consignee()
    if (have_consignee == 0 || have_consignee == '') {
        open_toast('请输入收货地址')
        return 0
    }
    var bonus = get_bonus()
    var bonus_sn = get_bonus_sn()
    var surplus = get_surplus()
    var shipping = get_shipping()
    for (x in shipping) {
        if (shipping[x] == 0) {
            var seller = $("#supplier_box_" + x).attr('seller')
            open_toast('请选择' + seller + '的配送方式')
            return false
        }
    }
    shipping = JSON.stringify(shipping)
    var pickup_point = get_pickup_point()
    var best_time = get_delivery_time()
    var need_inv = get_need_inv()
    var inv_type = get_inv_type()
    var inv_content = get_inv_content()
    var inv_payee_type = get_inv_payee_type()
    var inv_payee = get_inv_payee()
    var vat_inv_company_name = $('#vat_inv_company_name').val()
    var vat_inv_taxpayer_id = $('#vat_inv_taxpayer_id').val()
    var vat_inv_registration_address = $('#vat_inv_registration_address').val()
    var vat_inv_registration_phone = $('#vat_inv_registration_phone').val()
    var vat_inv_deposit_bank = $('#vat_inv_deposit_bank').val()
    var vat_inv_bank_account = $('#vat_inv_bank_account').val()
    var inv_consignee_name = $('#inv_consignee_name').val()
    var inv_consignee_phone = $('#inv_consignee_phone').val()
    var inv_consignee_country = "1"
    var inv_consignee_province = $('#inv_consignee_province').val()
    var inv_consignee_city = $('#inv_consignee_city').val()
    var inv_consignee_district = $('#inv_consignee_district').val()
    var inv_consignee_address = $('#inv_consignee_address').val()
    if (inv_type == 'vat_invoice') {
        if (vat_inv_company_name == '' || vat_inv_taxpayer_id == '' || vat_inv_registration_address == '' || vat_inv_registration_phone == '' || vat_inv_deposit_bank == '' || vat_inv_bank_account == '' || inv_consignee_name == '' || inv_consignee_phone == '' || inv_consignee_province == '' || inv_consignee_city == '' || inv_consignee_district == '' || inv_consignee_address == '') {
            open_toast('请填写增值税发票相关信息')
            return false
        }
    }
    var how_oos = get_how_oos()
    var postscript = $('#postscript').val()
    var payment = get_payment()
    var payment_other = get_payment_other()
    if (payment == 0 && payment_other == 0) {
        open_toast('请选择支付方式')
        return false
    }
    if (payment_other > 0) {
        payment = payment_other
    }
    ajax({
        data : {
            script_name : 'flow',
            step : 'done',
            have_consignee : have_consignee,
            bonus : bonus,
            bonus_sn : bonus_sn,
            surplus : surplus,
            pay_ship : shipping,
            best_time : best_time,
            need_inv : need_inv,
            inv_type : inv_type,
            inv_content : inv_content,
            inv_payee_type : inv_payee_type,
            inv_payee : inv_payee,
            vat_inv_company_name : vat_inv_company_name,
            vat_inv_taxpayer_id : vat_inv_taxpayer_id,
            vat_inv_registration_address : vat_inv_registration_address,
            vat_inv_registration_phone : vat_inv_registration_phone,
            vat_inv_deposit_bank : vat_inv_deposit_bank,
            vat_inv_deposit_bank : vat_inv_deposit_bank,
            vat_inv_bank_account : vat_inv_bank_account,
            inv_consignee_name : inv_consignee_name,
            inv_consignee_phone : inv_consignee_phone,
            inv_consignee_country : inv_consignee_country,
            inv_consignee_province : inv_consignee_province,
            inv_consignee_city : inv_consignee_city,
            inv_consignee_district : inv_consignee_district,
            inv_consignee_address : inv_consignee_address,
            how_oos : how_oos,
            postscript : postscript,
            payment : payment,
            payment_other : payment_other,
            pickup_point : pickup_point,
        },
        success : cb_submit_order,
    })
}

/**
 * @function
 * @name cb_submit_order
 * @description 提交订单回调函数
 * @param {string} data 服务器端返回的数据
 */
function cb_submit_order(data) {
    var json = parse_result(data)
    if (json) {
        $('body').html(json.content)
        publish(CHANNEL_ORDER_DONE)
        $('.to_order_list').on('click', function() {
            var composite_status = $(this).attr('composite_status') || '-1'
            back_to_user('open_order_list({composite_status : ' + composite_status + '})')
        })
        $('.payment').on('click', function() {
            open_pay($(this).attr('pay_online'))
        })
    }
}

/**
 * @function
 * @name open_surplus_verify
 * @description 打开余额支付密码验证弹窗
 */
function open_surplus_verify() {
    open_popup(surplus_verify_tmpl, ANIM_BOTTOM_TO_TOP, 'inline', false)

    $('#confirm_surplus_password').on('click', function() {
        verify_surplus_password()
    })

    $('#cancel_surplus_password').on('click', function() {
        close_popup()
    })
}

/**
 * @function
 * @name verify_surplus_password
 * @description 验证余额支付密码
 */
function verify_surplus_password() {
    var surplus_password = $('#surplus_password').val()
    if (surplus_password == '') {
        $('#surplus_password').focus()
        return false
    }

    ajax({
        data : {
            script_name : 'user',
            act : 'verify_surplus_password',
            surplus_password : surplus_password,
        },
        success : cb_verify_surplus_password,
        type : 'GET'
    })
}

/**
 * @function
 * @name cb_verify_surplus_password
 * @description 验证余额支付密码回调函数
 * @param {string} data 服务器端返回的数据
 */
function cb_verify_surplus_password(data) {
    if (data >= 1) {
        close_popup()
        submit_order()
    } else {
        open_toast('验证余额支付密码失败')
    }
}