/**
 * @description 每个页面的起始函数
 *              在appcan初始化原生插件之后执行
 */
appcan.ready(function() {
    del_val('opening_window')
    if (page_type == 'window' && page_name != 'root' && platform() == 'ios') {
        var status_bar_color = pop_val('status_bar_color')
        if (_.isEmpty(status_bar_color)) {
            status_bar_color = 1
        }
        uexWindow.setStatusBarTitleColor(status_bar_color)
    }
    appcan.window.subscribe(CHANNEL_GLOBAL, page_channel_func)
    if ($.isFunction(page_ready_func)) {
        var u = new Array()
        u.push(page_ready_func)
        page_ready_func = u
    }
    for (i in page_ready_func) {
        if ($.isFunction(page_ready_func[i])) {
            page_ready_func[i]()
        }
    }
})
/**
 * @type {object}
 * @description 全局变量的引用
 *              可以通过添加属性的方式定义全局变量
 *              例如：global.a = 'a'
 *              等价于 var a = 'a'
 */
var global = this

/**
 * @type {string}
 * @description 窗口类型，分为window和popover两种
 *              使用open_page打开的窗口为popover
 *              使用open_win打开的窗口为window
 *              判断是否为root窗口
 *              如果为root窗口，需要初始化page_type
 */
var page_type = pop_val('page_type')
if (_.isEmpty(page_type)) {
    page_type = 'window'
}
/**
 * @type {string}
 * @description 窗口名称
 *              默认是open_win和open_page函数的win_name参数
 *              判断是否为root窗口
 *              如果为root窗口，需要初始化page_name
 */
var page_name = pop_val('page_name')
if (_.isEmpty(page_name)) {
    page_name = 'root'
    add_val(KEY_PATH, 'root')
}
/**
 * @type {object}
 * @description 刷新用到的参数
 *              需要针对每个窗口进行配置
 */
var page_param = new Object()
/**
 * @type {object}
 * @description page_param的初始值
 *              使用$.extend(true)保证深度复制
 */
var page_param_org = $.extend(true, {}, page_param)

/**
 * @type {string}
 * @description 刷新用到的ajax方法
 *              默认为cb_reload
 *              在执行reload之前需要重新赋值
 *              因为有可能被其他js插件所改变
 */
var page_ajax_method = 'POST'

/**
 * @type {funcition}
 * @description 刷新用到的回调函数
 *              默认为cb_reload
 *              在执行reload之前需要重新赋值
 *              因为有可能被其他js插件所改变
 */
var page_callback = cb_reload

/**
 * @type {boolean}
 * @description 刷新时是否开启动画
 *              在执行reload之前需要重新赋值
 *              因为有可能被其他js插件所改变
 */
var page_loading = true

/**
 * @type {array}
 * @description 在init_common之前绑定的js事件
 */
var before_init_common = new Array()

/**
 * @type {object}
 * @description 错误代码对应的处理函数
 *              可以针对每个窗口进行配置
 */
var page_err_func = new Object()
/**
 * 各种错误的默认处理函数
 * 因为多个窗口都有可能出现以下错误
 * 所以写在main.js里
 * 最好针对每个窗口，分别配置
 */
/**
 * @type {function}
 * @description 需要登录
 */
page_err_func[ERR_NEED_LOGIN_FIRST] = function(json) {
    subscribe(CHANNEL_LOGIN_SUCCESS, function() {
        ajax({
            data : prev_ajax_data,
            success : prev_ajax_success,
            error : prev_ajax_error,
            type : prev_ajax_type,
        })
    }, "err_need_login_first")
    open_login()
    return false
}
/**
 * @type {function}
 * @description 商品缺货
 */
page_err_func[ERR_OUT_OF_STOCK] = function(json) {
    open_confirm({
        message : json.message[0],
        callback : function(index) {
            if (index == 1) {
                open_goods_booking(json.goods_id, json.goods_name)
            }
        }
    })
    return json
}
/**
 * @type {function}
 * @description 需要选择商品属性
 */
page_err_func[ERR_NEED_SELECT_ATTR] = function(json) {
    open_popup(json.content)
    init_select_attr_box(json.goods_id)
    return json
}
/**
 * @type {function}
 * @description 没有更多内容
 */
page_err_func[ERR_END_OF_LIST] = function(json) {
    open_toast(json.message)
    return json
}
/**
 * @type {object}
 * @description 窗口监听的事件所对应的函数
 */
var page_channel_func_arr = new Object()

/**
 * @type {array}
 * @description 每个窗口的默认起始函数
 *              即appcan.ready调用的函数
 *              默认为reload函数
 *              还可加入注册频道，初始化原生插件等任务
 */
var page_ready_func = [reload, init_bounce]

/**
 * @type {object}
 * @description 前一个ajax请求的参数
 */
var prev_ajax_data = new Object()
/**
 * @type {function}
 * @description 前一个ajax请求的成功回调函数
 */
var prev_ajax_success
/**
 * @type {function}
 * @description 前一个ajax请求的失败回调函数
 */
var prev_ajax_error
/**
 * @type {string}
 * @description 前一个ajax请求的请求方式
 */
var prev_ajax_type

/**
 * @function ajax
 * @description 发送ajax请求，对appcan.ajax的封装
 * @param {string} url 请求的地址
 * @param {object} data 参数
 * @param {function} success 请求成功回调函数
 * @param {function} error 请求错误回调函数
 * @param {string} type 请求方式，POST或者GET
 */
function ajax(url, data, success, error, type) {
    if (arguments.length === 1 && $.isPlainObject(url)) {
        data = url.data
        success = url.success
        error = url.error
        type = url.type
        url = url.url
    }
    var options = new Object()
    if ( typeof type != 'undefined') {
        options.type = type
    } else {
        options.type = 'POST'
    }
    if ( typeof url == 'undefined' || url == '') {
        url = controller_path()
    }
    options.appVerify = true
    options.url = url
    if (!$.isPlainObject(data)) {
        data = new Object()
    }
    options.data = data
    options.dataType = 'text'
    options.timeout = DUR_REQUEST_TIMEOUT

    options.success = function(data) {
        if ($.isFunction(success)) {
            success(data)
        }
        if ( typeof $.loading != 'undefined') {
            $.loading.end()
        }
        if ( typeof $.bounce != 'undefined') {
            $.bounce.end()
        }
    }

    options.error = function(xhr, type, error) {
        if ($.isFunction(error)) {
            error(xhr, type, error)
        } else {
            ajax_error(xhr, type, error)
        }
        if ( typeof $.loading != 'undefined') {
            $.loading.end()
        }
        if ( typeof $.bounce != 'undefined') {
            $.bounce.end()
        }
    }
    prev_ajax_data = data
    prev_ajax_success = success
    prev_ajax_error = error
    prev_ajax_type = type
    if (platform() == 'simulator') {
        options.headers = {
            'APPVERIFY' : get_verify()
        }
        $.ajax(options)
    } else {
        appcan.request.ajax(options)
    }
}

/**
 * @function get_verify
 * @description 获得加密字符串
 * @returns {string} verify
 */
function get_verify() {
    var ts = _.now().toString()
    var md5str = md5(APP_ID + ':' + APP_KEY + ':' + ts)
    var verify = 'md5=' + md5str + ';' + 'ts=' + ts + ';'
    return verify
}

/**
 * @function root_path
 * @description 入口链接
 * @param {string} path 在末尾拼接的字符串
 * @returns {string} _root_path
 */
function root_path(path) {
    var _root_path = ROOT_PATH
    if (DEBUG_MODE != DEBUG_NONE && !_.isEmpty(get_val(KEY_ROOT_PATH))) {
        _root_path = get_val(KEY_ROOT_PATH)
    }
    if ( typeof path == 'undefined') {
        path = ''
    }
    return _root_path + path
}

/**
 * @function app_root_path
 * @description 入口链接，即root_path()+"app/"
 * @param {string} path 在末尾拼接的字符串
 * @returns {string} _app_root_path
 */
function app_root_path(path) {
    var _app_root_path = root_path(APP_PATH + '/')
    if ( typeof path == 'undefined') {
        path = ''
    }
    return _app_root_path + path
}

/**
 * @function controller_path
 * @description 入口链接，即app_root_path()+controller.php
 * @param {string} path 在末尾拼接的字符串
 * @returns {string} _controller_path
 */
function controller_path(path) {
    var _controller_path = app_root_path('controller.php')
    if ( typeof path == 'undefined') {
        path = ''
    }
    return _controller_path + path
}

/**
 * @function ajax_error
 * @description 默认ajax失败回调函数
 * @param {object} xhr ajax对象
 * @param {string} type 错误类型
 * @param {string} error 错误代码
 */
function ajax_error(xhr, type, error) {
    var offline_tmpl = '<div class="offline _back"><div class="offline_bg">&nbsp</div>' + '<div class="offline_label">网络请求失败（<%= error_type %><%= error %>），请稍后重试，' + '<% if(page_name == "root"){ %>点击退出<% }else{ %>点击返回<% } %></div></div>'
    $('body').html(_.template(offline_tmpl)({
        page_name : page_name,
        error_type : type,
        error : error,
    }))
    init_common()
}

/**
 * @function reload
 * @description 窗口的刷新函数
 * @param {boolean} reset 是否重置窗口参数
 */
function reload(reset) {
    if (_.isEmpty(page_param_org) && !_.isEmpty(page_param)) {
        page_param_org = $.extend(true, {}, page_param)
    }
    if ( typeof reset != 'undefined' && reset === true) {
        page_param = $.extend(true, {}, page_param_org)
    }
    if ( typeof $.scroll != 'undefined') {
        $.scroll.enable()
    }
    if (!_.isEmpty(page_param)) {
        if (page_loading == true && typeof $.loading != 'undefined') {
            $.loading.start()
        }
        if ( typeof global.end_time != 'undefined') {
            clearTimeout(global.end_time)
        }
        ajax({
            data : page_param,
            success : page_callback,
            type : page_ajax_method,
        })
    }
}

/**
 * @function cb_reload
 * @description 默认Ajax回调函数
 * @param {string} data 服务器端返回的数据
 */
function cb_reload(data) {
    var json = parse_result(data)
    if (json && json.error <= 0) {
        $('body').children().removeClass('content_fade_in').addClass('content_fade_out')
        var content = json.content
        setTimeout(function() {
            $('body').html(content).children().addClass('content_fade_in')
            if ( typeof before_init_common == 'function') {
                before_init_common()
            }
            if ( typeof before_init_common == 'array') {
                for (i in before_init_common) {
                    if ( typeof before_init_common[i] == 'function') {
                        before_init_common[i]()
                    }
                }
            }
            init_common()
            init()
        }, DUR_ANIMATION)
    } else {
        subscribe(CHANNEL_ABORT_LOGIN, back, "err_need_login_first")
    }
}

/**
 * function open_win
 * @description 打开窗口（不含标题栏）
 * @param {string} win_url 窗口链接
 * @param {string} win_name 窗口名称
 * @param {string} anim_id 动画ID
 * @param {string} win_type 窗口类型
 */
function open_win(win_url, win_name, anim_id, win_type) {
    if (get_val('opening_window') === '1') {
        return false
    }
    if (arguments.length === 1 && $.isPlainObject(win_url)) {
        win_name = win_url.win_name
        anim_id = win_url.anim_id
        win_type = win_url.win_type
        win_url = win_url.win_url
    }
    if ( typeof win_name == 'undefined') {
        open_toast('请输入窗口名称')
        return false
    }
    if ( typeof win_type == 'undfined') {
        win_type = WIN_NONE
    }
    if (page_name == win_name) {
        return false
    }
    var path = JSON.parse(get_val(KEY_PATH))
    if ($.inArray(win_name, path) != -1) {
        back({
            target : win_name
        })
        return false
    }
    if ( typeof anim_id === 'undefined') {
        anim_id = ANIM_RIGHT_TO_LEFT
    }
    set_val('page_type', 'window')
    set_val('page_name', win_name)
    add_val(KEY_PATH, win_name)
    set_val('opening_window', '1')
    appcan.window.open(win_name, win_url, anim_id, win_type, 0, 0, DUR_ANIMATION)
}

/**
 * function open_page
 * @description 打开窗口（包含标题栏）
 * @param {string} win_url 窗口链接
 * @param {string} win_name 窗口名称
 * @param {string} win_title 窗口标题
 * @param {string} anim_id 动画ID
 * @param {string} win_type 窗口类型
 * @param {string} header 头部名称
 */
function open_page(win_url, win_name, win_title, anim_id, win_type, header) {
    if (arguments.length === 1 && $.isPlainObject(win_url)) {
        win_name = win_url.win_name
        win_title = win_url.win_title
        anim_id = win_url.anim_id
        win_type = win_url.win_type
        header = win_url.header
        win_url = win_url.win_url
    }
    if ( typeof win_url == 'undefined' || typeof win_name == 'undefined') {
        appcan.window.alert('错误', 'open_page参数错误,win_url=' + win_url + ',win_name=' + win_name)
        return false
    }
    if (_.isUndefined(win_type)) {
        win_type = WIN_NONE
    }
    if (_.isUndefined(anim_id)) {
        anim_id = ANIM_RIGHT_TO_LEFT
    }
    if ( typeof header == 'undefined') {
        header = 'common_header'
    }
    set_val('header', header)
    set_val('win_url', win_url)
    set_val('win_name', win_name)
    set_val('win_title', win_title)
    open_win('common_page.html', win_name, anim_id, win_type)
}

/**
 * function open_toast
 * @description 打开提示
 * @param {string} content 内容
 * @param {string} duration 显示时间
 * @param {string} position 位置，默认为"5"
 * @param {string} type 类型,默认为"0"
 */
function open_toast(content, duration, position, type) {
    if (arguments.length === 1 && $.isPlainObject(content)) {
        duration = content.duration
        position = content.position
        type = content.type
        content = content.content
    }
    if ( typeof content === 'undefined') {
        return false
    }
    if ( typeof content == 'array') {
        content = content.join('，')
    }
    if ( typeof content == 'string' && content.length == 0) {
        return false
    }
    if ( typeof duration == 'undefined') {
        duration = DUR_TOAST
    }
    if ( typeof position == 'undefined') {
        position = '5'
    }
    if ( typeof type == 'undefined') {
        type = '0'
    }
    appcan.window.openToast(content, duration, position, type)
}

/**
 * @function close_toast
 * @description 关闭所有提示
 */
function close_toast() {
    appcan.window.closeToast()
}

/**
 * function open_confirm
 * @description 打开确认界面
 * @param {string} title 标题
 * @param {string} message 消息
 * @param {string|array} buttons 弹窗包含的按钮
 *                              如果为string，用","链接
 * @param {function} callback 点击按钮回调函数
 */
function open_confirm(title, message, buttons, callback) {
    if (arguments.length === 1 && $.isPlainObject(title)) {
        message = title.message
        buttons = title.buttons
        callback = title.callback
        title = title.title
    }
    if ( typeof buttons === 'undefined' || buttons == '') {
        buttons = ['取消', '确定']
    }
    if ( typeof buttons == 'array') {
        buttons = buttons.join(',')
    }
    if ( typeof title === 'undefined' || title == '') {
        title = '提示'
    }
    appcan.window.confirm(title, message, buttons, function(err, data, dataType, opId) {
        callback(data)
    })
}

/**
 * function open_goods
 * @description 打开商品详情
 * @param {string} goods_id 商品ID
 */
function open_goods(goods_id) {
    set_val('goods_id', goods_id)
    open_win('goods.html', 'goods_' + goods_id)
}

/**
 * function open_goods_list
 * @description 打开商品列表
 * @param {string} cat_id 分类ID
 * @param {string} supplier_id 店铺ID
 * @param {string} keywords 关键词
 */
function open_goods_list(cat_id, supplier_id, keywords) {
    if ($.isPlainObject(cat_id) && arguments.length == 1) {
        for (x in cat_id) {
            set_val(x, cat_id[x])
        }
        supplier_id = cat_id.supplier_id
    } else {
        set_val('cat_id', cat_id)
        set_val('supplier_id', supplier_id)
        set_val('keywords', keywords)
    }
    if ( typeof supplier_id == 'undefined' || supplier_id == "") {
        supplier_id = 0
    } else {
        supplier_id = parseInt(supplier_id)
    }
    open_win('goods_list.html', 'goods_list_' + supplier_id.toString())
}

/**
 * function add_to_cart
 * @description 加入购物车
 * @param {string} goods_id 商品ID
 * @param {number} goods_number 商品数量
 * @param {array} spec_arr 商品属性
 * @param {string} quick 快速购买
 * @param {string} parent_id 商品的parent_id
 * @param {string} one_step_buy 立即购买
 */
function add_to_cart(goods_id, goods_number, spec_arr, quick, parent_id, one_step_buy) {
    if (arguments.length === 1 && $.isPlainObject(goods_id)) {
        goods_number = goods_id.goods_number
        spec_arr = goods_id.spec_arr
        quick = goods_id.quick
        parent_id = goods_id.parent_id
        one_step_buy = goods_id.one_step_buy
        goods_id = goods_id.goods_id
    }
    if ( typeof goods_number === 'undefined') {
        goods_number = 1
    }
    if ( typeof one_step_buy == 'undefined') {
        one_step_buy = 0
    }
    set_val('one_step_buy', one_step_buy)
    if ( typeof spec_arr === 'undefined') {
        spec_arr = new Array()
    }

    if ( typeof quick === 'undefined') {
        quick = 0
    }

    if ( typeof parent_id === 'undefined') {
        parent_id = 0
    }

    var goods = new Object()
    goods.quick = quick
    goods.parent = parent_id
    goods.spec = spec_arr
    goods.goods_id = goods_id
    goods.number = goods_number
    ajax({
        data : {
            script_name : 'flow',
            step : 'add_to_cart',
            goods : JSON.stringify(goods),

        },
        success : cb_add_to_cart,
        type : 'POST',
    });
}

/**
 * function cb_add_to_cart
 * @description 加入购物车回调函数
 * @param {string} data 服务器端返回的数据
 */
function cb_add_to_cart(data) {
    var json = parse_result(data)
    if (json.error == ERR_OVER_BUY_MAX) {
        $('#goods_number_' + json.goods_id).val(json.num_else)
        $('#plus_' + json.goods_id).attr('buy_max', json.num_else).addClass('disabled')
    } else if (json.error == ERR_NEED_LOGIN_TO_BUY) {
        open_confirm('', '此商品为限购商品，是否登录购买', ['算了', '去登录'], function(data) {
            if (data == 1) {
                subscribe(CHANNEL_LOGIN_SUCCESS, reload, "err_need_login_to_buy")
                open_login()
            }
        })
    } else if (json.error <= 0) {
        one_step_buy = pop_val('one_step_buy')
        if (one_step_buy == 1) {
            var selected_goods = new Array()
            var goods_number = new Object()
            selected_goods.push(json.rec_id)
            goods_number[json.rec_id] = 1
            set_val('selected_goods', JSON.stringify(selected_goods))
            set_val('goods_number', JSON.stringify(goods_number))
            open_page('checkout.html', 'checkout', '结算')
        } else {
            if ( typeof go_to_cart_tmpl == 'undefined') {
                open_toast('成功加入购物车')
            } else {
                open_popup({
                    content : _.template(go_to_cart_tmpl)({
                        message : json.message
                    }),
                    anim_id : ANIM_BOTTOM_TO_TOP,
                    close_button : false,
                })
                $('#close_popup').on('click', close_popup)
                $('#go_to_cart').on('click', function() {
                    back_to_cart()
                    close_popup()
                })
            }
        }
        publish([CHANNEL_UPDATE_CART, CHANNEL_ADD_TO_CART], json.goods_number)
    }
}

/**
 * function init_select_attr_box
 * @description 初始化商品属性选择弹窗
 * @param {string} g 商品ID
 */
function init_select_attr_box(g) {
    var target = $('#goods_attr_container_' + g)
    var goods_id = $(target).attr('goods_id')
    if (g != goods_id) {
        open_toast('商品ID不匹配')
        return false
    }
    var parent_id = $(target).attr('parent_id')
    $(target).find('.goods_attr_container').each(function() {
        var attr_container = this
        $(attr_container).find('.goods_attr').each(function(i) {
            $(this).on('click', function() {
                $(attr_container).find('.goods_attr').each(function(j) {
                    $(this).toggleClass('selected', i == j)
                })
            })
        })
    })

    $(target).find('.confirm_attr_button').on('click', function() {
        confirm_select_goods_attr(goods_id)
        close_popup()
    })
}

/**
 * function confirm_select_goods_attr
 * @description 确认商品属性
 * @param {string} g 商品ID
 */
function confirm_select_goods_attr(g) {
    var target = $('#goods_attr_container_' + g)
    var goods_id = $(target).attr('goods_id')
    if (g != goods_id) {
        open_toast('商品ID不匹配')
        return false
    }
    var parent_id = $(target).attr('parent_id')
    var quick = 1
    var spec = new Array()
    $(target).find('.goods_attr_container').each(function(i) {
        spec[i] = $(this).find('.goods_attr').not(function() {
            return !$(this).hasClass('selected')
        }).attr('attr_id')
    })
    var goods_number = 1
    if ($('#goods_number_' + goods_id).length > 0) {
        goods_number = $('#goods_number_' + goods_id).val()
    }
    add_to_cart(goods_id, goods_number, spec, quick, parent_id)
}

/**
 * function open_cart
 * @description 打开购物车
 *              购物车在不同的位置打开具有不同的头部
 * @param {string} header 头部名称
 */
function open_cart(header) {
    if ( typeof header == 'undefined') {
        header = 'common_header'
    }
    open_page({
        win_url : 'cart.html',
        win_name : 'cart',
        win_title : '购物车',
        anim_id : '12',
        header : header,
    })
}

/**
 * function subscribe
 * @description 订阅一个频道，如果有消息发给该频道，则会执行响应的回调
 * @param {string|array} channel_id 频道ID
 * @param {function|array} callback 响应函数
 * @param {string|array} unique_id 唯一标识，相同标识的订阅会被覆盖
 */
function subscribe(channel_id, callback, unique_id) {
    if (_.isArray(channel_id)) {
        for (var i in channel_id) {
            subscribe(channel_id[i], callback, unique_id)
        }
        return false
    }
    if ( typeof unique_id === 'undefined') {
        unique_id = unique_page_id()
    }
    if ( typeof page_channel_func_arr[channel_id] === 'undefined') {
        page_channel_func_arr[channel_id] = {}
    }
    page_channel_func_arr[channel_id][unique_id] = callback
}

/**
 * function publish
 * @description 向指定通道发送消息
 * @param {string|array} channel_id 频道ID
 * @param {string|array} msg 消息内容
 */
function publish(channel_id, msg) {
    if (_.isArray(channel_id)) {
        if (!_.isArray(msg)) {
            msg = [msg]
        }
        for (i in channel_id) {
            if ( typeof msg[i] != 'undefined') {
                publish(channel_id[i], msg[i])
            } else {
                publish(channel_id[i], msg[0])
            }
        }
    } else {
        var tmp = {
            channel_id : channel_id,
            page_name : page_name,
            page_type : page_type,
        }

        if ($.isPlainObject(msg) || $.isArray(msg)) {
            for (i in msg) {
                tmp[i] = msg[i]
            }
        } else {
            tmp['_msg'] = msg
        }
        appcan.window.publish(CHANNEL_GLOBAL, JSON.stringify(tmp))
    }
}

/**
 * function save_goods_history
 * @description 存储商品浏览记录
 * @param {object} data
 */
function save_goods_history(data) {
    if (!get_val(KEY_GOODS_HISTORY)) {
        var goods_history = new Object()
    } else {
        var goods_history = JSON.parse(get_val(KEY_GOODS_HISTORY))
        if (goods_history.length > 1000) {
            open_toast('浏览记录超过1000条，无法添加更多记录')
        }
    }
    if ( typeof (goods_history[data.goods_id]) != 'undefined') {
        data['visit_count'] = goods_history[data.goods_id]['visit_count'] + 1
    } else {
        data['visit_count'] = 1
    }
    data['last_visit'] = _.now()
    goods_history[data.goods_id] = data
    set_val(KEY_GOODS_HISTORY, JSON.stringify(goods_history))
}

/**
 * function clear_goods_history
 * @description 删除所有浏览记录
 */
function clear_goods_history() {
    appcan.locStorage.remove(KEY_GOODS_HISTORY);
}

/**
 * function delete_goods_history
 * @description 删除商品浏览记录
 * @param {string} goods_id 商品ID
 */
function delete_goods_history(goods_id) {
    if (get_val(KEY_GOODS_HISTORY)) {
        var goods_history = JSON.parse(get_val(KEY_GOODS_HISTORY));
        if ( typeof (goods_history[goods_id]) != 'undefined') {
            delete goods_history[goods_id];
        }
        set_val(KEY_GOODS_HISTORY, JSON.stringify(goods_history));
    }
}

/**
 * function get_goods_history
 * @description 查找商品浏览记录
 * @param {object} filter
 * @returns {object} tmp_result 商品历史记录
 */
function get_goods_history(filter) {
    if (_.isUndefined(filter)) {
        filter = new Object();
    }
    if (_.isUndefined(filter.page)) {
        var page = 1;
    } else {
        var page = filter.page;
    }
    if (_.isUndefined(filter.size)) {
        var size = 10;
    } else {
        var size = filter.size;
    }
    if (_.isUndefined(filter.sort_by)) {
        var sort_by = 'last_visit';
    } else {
        var sort_by = filter.sort_by;
    }
    if (_.isUndefined(filter.order)) {
        var order = 'DESC';
    } else {

        var order = filter.order.toUpperCase();
    }

    if (!get_val(KEY_GOODS_HISTORY)) {
        var goods_history = new Object();
    } else {
        var goods_history = JSON.parse(get_val(KEY_GOODS_HISTORY));
    }

    var start = (page - 1) * size;
    var end = start + size;
    var tmp_result = _.sortBy(goods_history, function(data) {
        if (order == 'DESC') {
            return -data[sort_by]
        } else {
            return data[sort_by]
        }
    }).slice(start, end)

    for (i in tmp_result) {
        var commonTime = new Date(tmp_result[i]['last_visit']).toLocaleString()
        tmp_result[i]['formatted_last_visit'] = commonTime
    }
    return tmp_result;
}

/**
 * function open_search
 * @description 打开搜索界面
 */
function open_search() {

    open_win('search.html', 'search', ANIM_BOTTOM_TO_TOP);
}

/**
 * function open_search_result
 * @description 打开搜索结果页面
 * @param {string} keywords 关键词
 */
function open_search_result(keywords) {
    set_val('keywords', keywords)
    set_val('script_name', 'goods_list')
    open_win('goods_list.html', 'goods_list')
}

/**
 * function save_search_history
 * @description 存储搜索记录
 * @param {string} keywords 关键词
 * @param {string} type 搜索类型
 */
function save_search_history(keywords, type) {
    if (_.isEmpty(keywords)) {
        return false
    }

    if (_.isUndefined(type)) {
        type = 0
    }

    var search_history = get_val(KEY_SEARCH_HISTORY);
    if (_.isString(search_history) && search_history != '') {
        search_history = JSON.parse(search_history);
    } else {
        search_history = new Object();
        search_history[SEARCH_HISTORY_GOODS] = new Array();
        search_history[SEARCH_HISTORY_SHOP] = new Array();
        search_history[SEARCH_HISTORY_HOT] = new Array();
    }

    if (_.isString(keywords)) {
        keywords = [keywords];
    }
    for (i in keywords) {
        if (_.isEmpty(keywords[i])) {
            continue
        }
        var stored = false;
        for (j in search_history[type]) {
            if (search_history[type][j]['keywords'] == keywords[i]) {
                search_history[type][j]['count']++;
                stored = true;
            }
        }
        if (!stored) {
            search_history[type].push({
                keywords : keywords[i],
                count : 1
            });
        }
    }
    set_val(KEY_SEARCH_HISTORY, JSON.stringify(search_history));
    return true;
}

/**
 * @function get_search_history
 * @description 取得搜索记录
 *              如果不提供搜索类型
 *              则返回所有搜索历史记录
 * @param {string} type 搜索类型
 * @returns {array} search_history 搜索记录
 */
function get_search_history(type) {
    var search_history = get_val(KEY_SEARCH_HISTORY);
    if (_.isString(search_history) && search_history != '') {
        search_history = JSON.parse(search_history);
        if (_.isUndefined(type)) {
            return search_history;
        } else {
            return search_history[type];
        }

    } else {
        search_history = new Object();
        search_history[SEARCH_HISTORY_GOODS] = new Array();
        search_history[SEARCH_HISTORY_SHOP] = new Array();
        search_history[SEARCH_HISTORY_HOT] = new Array();
        return search_history;
    }
}

/**
 * @function set_search_history
 * @description 存储历史纪录到缓存中
 * @param {object} search_history
 */
function set_search_history(search_history) {
    if ($.isPlainObject(search_history)) {
        search_history = JSON.stringify(search_history)
    }
    set_val(KEY_SEARCH_HISTORY, search_history)
}

/**
 * @function delete_search_history
 * @description 删除搜索记录
 *              如果不提供搜索类型
 *              则删除所有可删除的搜索记录
 * @param {string} type 搜索类型
 */
function delete_search_history(type) {
    var search_history = get_search_history()
    search_history[type] = []
    set_search_history(search_history)
}

/**
 * @function open_scan
 * @description 打开扫描窗口
 */
function open_scan() {
    uexScanner.cbOpen = cb_scan
    uexDevice.cbGetInfo = function(opCode, dataType, data) {
        var device = JSON.parse(data)
        var camera = device.camera
        if (camera != null && camera.length > 0) {
            if (camera == 1) {
                uexScanner.open()
            }
        }
    }
    uexDevice.getInfo(DEVICE_CAMERA)
}

/**
 * @function cb_scan
 * @description 扫描完成后的回调函数
 * @param {string} opId
 * @param {string} dataType
 * @param {string} data
 */
function cb_scan(opId, dataType, data) {
    var json = JSON.parse(data)
    var type = json.type
    var code = json.code
    //条形码
    if (type == 'EAN_13' || type == 'EAN-13') {
        var buttons = ['取消', '复制', '搜索']
        open_confirm('条形码扫描', '条形码内容为：' + code + '，选择您要进行的操作', buttons, function(data) {
            if (data == 1) {
                uexClipboard.copy(code)
            } else if (data == 2) {
                open_barcode_result(code)
            }
        })
    }
    //二维码
    else if (type == 'QR_CODE' || type == 'QR Code') {
        test_qrcode_result(code)
    }
}

/**
 * @function test_qrcode_result
 * @description 验证二维码信息
 *              如果为本店商品信息，则打开商品详情
 *              如果为店铺信息，则打开店铺详情
 *              如果未连接，则提示是否打开链接
 *              还提供复制功能
 * @param {string} code 二维码内容
 */
function test_qrcode_result(code) {
    if (REG_GOODS_LINK.test(code) == true) {
        open_goods(code.replace(REG_GOODS_LINK, '$2'))
    } else if (REG_SUPPLIER_LINK.test(code) === true) {
        open_supplier(code.replace(REG_SUPPLIER_LINK, '$2'))
    } else {
        var buttons = ['取消', '复制']
        if (code.indexOf('http://') === 0 || code.indexOf('https://') === 0) {
            buttons.push('打开')
        }
        open_confirm('二维码扫描', '二维码内容为：' + code + '，选择您要进行的操作', buttons, function(data) {
            if (data == 1) {
                uexClipboard.copy(code)
            } else if (data == 2) {
                open_app_url(code, '二维码链接')
            }
        })
    }
}

/**
 * @function open_browser
 * @description 打开连接
 * @param {string} url 链接
 * @param {string} title 标题
 * @param {string} type 打开方式
 */
function open_browser(url, title, type) {
    if ($.isPlainObject(url) && arguments.length == 1) {
        title = url.title
        type = url.type
        url = url.url
    }

    if ( typeof title == 'undefined') {
        title = '链接'
    }
    if ( typeof type == 'undefined') {
        type = TYPE_BROWSER
    }
    if (type == TYPE_BROWSER) {
        switch(platform()) {
        case 'android':
            uexWidget.startApp("1", "android.intent.action.VIEW", '{"data":{"mimeType":"text/html","scheme":"' + url + '"}}')
            break
        case 'ios':
            uexWidget.loadApp(url, '', '')
            break
        case 'simulator':
            open_page(url, 'url', title)
            break
        default:
            open_page(url, 'url', title)
            break
        }
    } else if (type == TYPE_APP) {
        open_page(url, 'url', title)
    }
}

/**
 * @function open_app_url
 * @description 在应用内打开链接
 * @param {string} url 链接
 * @param {string} title 标题
 */
function open_app_url(url, title) {
    if ( typeof title == 'undefined') {
        title = '链接'
    }
    open_page(url, _.uniqueId('url_'), title)
}

/**
 * @function platform
 * @description 获取平台
 * @returns {string} platform（小写）
 */
function platform() {
    return uexWidgetOne.platformName.toLowerCase()
}

/**
 * @function start_count_down
 * @description 开始倒计时
 * @param {string} id 元素ID
 * @param {number} count 倒计时
 * @param {number} origin_count 初始倒计时
 * @param {string} origin_html 初始文本
 */
function start_count_down(id, count, origin_count, origin_html) {
    var target = $('#' + id)
    if (count == 0) {
        $(target).html(origin_html)
        $(target).data('count', origin_count)
        $(target).removeClass('disabled')
        return false
    }
    $(target).addClass('disabled')
    count = parseInt(count)
    $(target).html(count + '秒后可再次发送')
    count--
    $(target).data('count', count)
    setTimeout(function() {
        start_count_down(id, count, origin_count, origin_html)
    }, DUR_SECOND)
}

/**
 * @function open_user_message
 * @description 打开发布留言的窗口
 */
function open_user_message() {
    open_page({
        win_url : 'user_message.html',
        win_name : 'user_message',
        win_title : '发布留言',
    })
}

/**
 * @function open_article
 * @description 打开文章
 * @param {string} article_id 文章ID
 */
function open_article(article_id) {
    set_val('article_id', article_id)
    open_page('article.html', 'article', '文章详情')
}

/**
 * @function parse_result
 * @description 转换服务器返回的数据
 *              如果是{error:0,message:'',content:''}的格式
 *              自动处理错误
 * @param {string} data 服务器端返回的数据
 */
function parse_result(data) {
    try {
        var json = eval('(' + data + ')');
    } catch(ex) {
        $('body').html(data)
        return false
    }
    if (json.error > 0) {
        if ( typeof page_err_func[ERR_ALL] != 'undefined') {
            return page_err_func[ERR_ALL](json)
        } else if ( typeof page_err_func[json.error] != 'undefined') {
            return page_err_func[json.error](json)
        } else if ( typeof page_err_func[ERR_OTHERS] != 'undefined') { typeof page_err_func[ERR_OTHERS](json)
        } else {
            open_toast(json.message)
        }
    } else {
        for (x in json) {
            switch(x) {
            case 'message':
            case 'content':
            case 'error':
                break
            case 'search_keywords':
                save_search_history(json[x].split(','), SEARCH_HISTORY_HOT)
                break
            case 'cart_num':
                publish(CHANNEL_UPDATE_CART, json[x])
                break
            case 'app_param':
                for (xx in json[x]) {
                    global[xx] = json[x][xx]
                }
                break
            default:
                set_val(x, json[x])
                break
            }
        }
    }
    return json
}

/**
 * @function open_goods_comment
 * @description 打开评价详情
 * @param {string} rec_id 评价唯一标识
 */
function open_goods_comment(rec_id) {
    set_val('rec_id', rec_id)
    open_page('goods_comment.html', 'goods_comment', '评价详情');
}

/**
 * @function open_goods_shaidan
 * @description 打开晒单详情
 * @param {string} shaidan_id 晒单唯一标识
 */
function open_goods_shaidan(shaidan_id) {
    set_val('shaidan_id', shaidan_id)
    open_page('goods_shaidan.html', 'goods_shaidan', '晒单详情');
}

/**
 * @function open_add_comment
 * @description 添加评价
 * @param {string} rec_id 评价唯一标识
 * @param {string} goods_id 商品ID
 */
function open_add_comment(rec_id, goods_id) {
    set_val('rec_id', rec_id)
    set_val('goods_id', goods_id)
    open_page('user_add_comment.html', 'user_add_comment', '发表评价');
}

/**
 * @function open_add_shaidan
 * @description 添加晒单
 * @param {string} rec_id 晒单唯一标识
 * @param {string} goods_id 商品ID
 */
function open_add_shaidan(rec_id, goods_id) {
    set_val('rec_id', rec_id)
    set_val('goods_id', goods_id)
    open_page('user_shaidan_send.html', 'user_shaidan_send', '发表晒单');
}

/**
 * @function open_supplier
 * @description 打开店铺首页
 * @param {string} supplier_id 店铺唯一标识
 * @param {string} shop_name 店铺名称
 */
function open_supplier(supplier_id, shop_name) {
    if (arguments.length === 1 && $.isPlainObject(supplier_id)) {
        shop_name = supplier_id.shop_name
        supplier_id = supplier_id.supplier_id
    }
    set_val('supplier_id', supplier_id)
    open_win('supplier_index.html', 'supplier_index')
}

/**
 * @function add_val
 * @description 打开店铺分类列表
 * @param {string} supplier_id 店铺唯一标识
 * @param {string} shop_name 店铺名称
 */
function open_category(supplier_id, shop_name) {
    if ( typeof supplier_id == 'undefined') {
        supplier_id = 0
    }
    if ( typeof shop_name == 'undefined') {
        shop_name = '店铺'
    }
    set_val('supplier_id', supplier_id)
    open_page('category.html', 'category_' + supplier_id, shop_name + '分类')
}

/**
 * @function add_val
 * @description 打开子分类列表
 * @param {string} supplier_id 店铺唯一标识
 * @param {string} shop_name 店铺名称
 */
function open_category_son(cat_id,cat_name) {
    set_val('cat_id', cat_id)
    open_page('category_son.html', 'category_son_' + cat_id, cat_name)
}

/**
 * @function add_val
 * @description 在缓冲中存储堆栈数据
 *              如果val已经存在，会清除其后的数据
 *              目前只有路径（_path）用到这个方法
 * @param {string} key 键
 * @param {string} val 值
 */
function add_val(key, val) {
    if ( typeof localStorage[key] === 'undefined') {
        var arr = new Array()
    } else {
        var arr = JSON.parse(localStorage[key])
    }
    var is_exist = false
    for (i in arr) {
        if (_.isEqual(arr[i], val)) {
            is_exist = true
            arr = arr.slice(0, i + 1)
            break
        }
    }
    if (!is_exist) {
        arr.push(val)
    }
    localStorage[key] = JSON.stringify(arr)
}

/**
 * @function rmv_val
 * @description 清除缓存中的堆栈式数据
 *              即清除val及其后的数据
 *              目前只有路径（_path）用到这个方法
 * @param {string} key 键
 * @param {string} val 值
 */
function rmv_val(key, val) {
    if ( typeof localStorage[key] === 'undefined') {
        var arr = new Array()
        localStorage[key] = JSON.stringify(arr)
        return
    }
    var arr = JSON.parse(localStorage[key])
    var target_position = $.inArray(val, arr)
    if (target_position != -1) {
        arr.splice(target_position, arr.length - target_position)
    }
    localStorage[key] = JSON.stringify(arr)
}

/**
 * @function set_val
 * @description 存储值
 *              如果值为null、NaN、undefined
 *              则不存储
 * @param {string} key 键
 * @param {string} val 值
 */
function set_val(key, val) {
    if (!_.isNaN(val) && !_.isNull(val) && !_.isUndefined(val)) {
        localStorage[key] = val
    }
}

/**
 * @function get_val
 * @description 获取缓存中的值
 * @param {string} key 键
 * @return {string} localStorage[key] 如果不存在则返回 空字符串
 */
function get_val(key) {
    if (!_.isNaN(localStorage[key]) && !_.isNull(localStorage[key]) && !_.isUndefined(localStorage[key])) {
        return localStorage[key]
    } else {
        return ''
    }
}

/**
 * @function del_val
 * @description 删除缓存的值
 * @param {string} key 键
 */
function del_val(key) {
    delete localStorage[key]
}

/**
 * @function pop_val
 * @description 获取缓存中的值，并删除
 * @param {string} key 键
 * @return {string} tmp 值
 */
function pop_val(key) {
    var tmp = get_val(key)
    del_val(key)
    return tmp
}

/**
 * @function eval_script
 * @description 指定窗口执行指定的代码
 * @param {!string|array} win_name 执行script的窗口名称
 * @param {string} script 要执行的代码
 * @param {string} type 窗口类型（window、popover）
 * @param {?string|array} pop_name 如果窗口类型为popover
 *                          且没有传入此参数
 *                          则默认使用win_name作为弹窗的名称
 */
function eval_script(win_name, script, type, pop_name) {
    if (arguments.length === 1 && $.isPlainObject(win_name)) {
        script = win_name.script
        type = win_name.type
        pop_name = win_name.pop_name
        win_name = win_name.win_name
    }
    if ( typeof type == 'undefined') {
        type = 'window'
    }
    if (_.isString(win_name) && win_name.length == 0) {
        return
    }
    if (_.isString(win_name)) {
        win_name = [win_name]
    }

    if (type === 'popover' && typeof pop_name == 'undefined') {
        pop_name = win_name
    }
    if (type === 'popover' && _.isString(pop_name)) {
        pop_name = [pop_name]
    }
    for (i in win_name) {
        if (type === 'window') {
            appcan.window.evaluateScript(win_name[i], script)
        } else if (type === 'popover') {
            appcan.window.evaluatePopoverScript(win_name[i], pop_name[i], script)
        }
    }
}

/**
 * @function back
 * @description 后退到指定窗口
 *              默认后退到上一个窗口
 * @param {string} anim_id 动画ID
 * @param {number} duration 动画时间
 * @param {string} target 目标窗口名称
 */
function back(anim_id, duration, target) {
    if (arguments.length === 1 && $.isPlainObject(anim_id)) {
        duration = anim_id.duration
        target = anim_id.target
        anim_id = anim_id.anim_id
    }
    //如果是root窗口
    if (page_type == 'window' && page_name == 'root') {
        if ($.back >= 1) {
            uexWidgetOne.exit(0)
        } else {
            $.back = 1
            open_toast('再次点击返回键，退出程序', DUR_BACK)
            clearTimeout($.back_t)
            $.back_t = setTimeout(function() {
                $.back = 0
            }, DUR_BACK)
            return false
        }
    } else {
        if ( typeof anim_id === 'undefined') {
            anim_id = -1
        }
        if ( typeof duration === 'undefined') {
            duration = DUR_ANIMATION
        }
        if ( typeof target === 'undefined') {
            if (page_type == 'popover') {
                eval_script(page_name, 'back(' + anim_id + ', ' + duration + ')')
            } else {
                rmv_val(KEY_PATH, page_name)
                appcan.window.close(parseInt(anim_id), parseInt(duration))
            }
        } else {
            if (page_type == 'popover') {
                eval_script(page_name, 'back(' + anim_id + ', ' + duration + ',\'' + target + '\')')
            } else {
                var path = JSON.parse(get_val(KEY_PATH))
                var target_position = $.inArray(target, path)
                if (target_position != -1) {
                    var after_target = path.slice(target_position + 1, path.length - 1)
                    eval_script(after_target, 'back(0,0)')
                    back()
                }
            }
        }
    }

}

/**
 * @function open_goods_booking
 * @description 缺货登记
 * @param {string} goods_id 商品ID
 * @param {string} goods_name 商品名称
 * @param {string} goods_img 商品图片
 */
function open_goods_booking(goods_id, goods_name, goods_img) {
    if (arguments.length === 1 && $.isPlainObject(goods_id)) {
        goods_name = goods_id.goods_name
        goods_img = goods_id.goods_img
        goods_id = goods_id.goods_id
    }
    set_val('goods_id', goods_id)
    set_val('goods_name', goods_name)
    set_val('goods_img', goods_img)
    book_goods(goods_id)
}

/**
 * @function book_goods
 * @description 缺货登记
 * @param {string} goods_id 商品ID
 */
function book_goods(goods_id) {
    if ( typeof $.loading != 'undefined') {
        $.loading.start()
    }
    ajax({
        data : {
            script_name : 'user',
            act : 'book_goods',
            id : goods_id,
            no_have : '0',
        },
        success : cb_book_goods,
        type : 'GET',
    })
}

/**
 * @function cb_book_goods
 * @description 缺货登记回调函数
 * @param {string} data 服务器端返回的数据
 */
function cb_book_goods(data) {
    var json = parse_result(data)
    if (json && json.error <= 0) {
        var email = json.email
        var goods_id = json.goods_id
        var no_have = json.no_have
        var tel = json.tel
        set_val('email', email)
        set_val('goods_id', goods_id)
        set_val('tel', tel)
        open_page('goods_booking.html', 'goods_booking', '缺货登记')
    }
}

/**
 * @function open_popup
 * @description 打开弹窗
 * @param {string} content 弹窗内容，可以为选择器，html内容，或zepto对象
 * @param {string} anim_id 动画ID
 * @param {string} type 弹窗类型
 *                      详见：http://dimsemenov.com/plugins/magnific-popup/documentation.html
 * @param {boolean} close_button 是否包含关闭按钮
 * @param {function } callback 回调函数
 * @param {boolean} close_on_bg_click 点击背景是否关闭
 */
function open_popup(content, anim_id, type, close_button, callback, close_on_bg_click) {
    if (arguments.length === 1 && $.isPlainObject(content)) {
        anim_id = content.anim_id
        type = content.type
        close_button = content.close_button
        callback = content.callback
        close_on_bg_click = content.close_on_bg_click
        content = content.content
    }
    if ( typeof anim_id === 'undefined' || anim_id === '') {
        anim_id = ANIM_BOTTOM_TO_TOP
    }

    if ( typeof callback == 'undefined') {
        if ( typeof $.bounce != 'undefined') {
            callback = {
                open : $.bounce.disable,
                close : $.bounce.enable,
            }
        } else {
            callback = {
                open : function() {
                },
                close : function() {
                }
            }
        }
    }

    var data = new Object()
    switch(anim_id) {
    case ANIM_BOTTOM_TO_TOP:
        data.mainClass = 'my-mfp-slide-bottom'
        break
    case ANIM_RIGHT_TO_LEFT:
        data.mainClass = 'my-mfp-slide-right'
        break
    }
    if ( typeof type === 'undefined') {
        data.type = 'inline'
    } else {
        data.type = type
    }
    if ( typeof close_button == 'undefined') {
        close_button = true
    }
    if ( typeof callback == 'undefined') {
        callback = {}
    }
    if ( typeof close_on_bg_click == 'undefined') {
        close_on_bg_click = true
    }
    data.callbacks = callback
    data.showCloseBtn = close_button
    data.closeOnBgClick = close_on_bg_click
    data.items = {
        src : content
    }
    data.removalDelay = DUR_ANIMATION
    if ( typeof name != 'undefined') {
        data.key = name
    }

    $.magnificPopup.open(data)
}

/**
 * @function close_popup
 * @description 关闭弹窗
 */
function close_popup() {
    $.magnificPopup.close()
}

/**
 * @function get_theme_file
 * @description 根据当前主题获取模板文件
 *              只能从根目录的html文件中调用
 *              只能请求themes路径下的文件
 * @param {string} file 文件名称
 */
function get_theme_file(file) {
    if (_.isArray(file)) {
        for (i in file) {
            get_theme_file(file[i])
        }
    } else {
        file_path = 'themes/' + THEME + '/' + file
        get_script_sync(file_path)
    }
}

/**
 * @function open_login
 * @description 打开登录窗口
 */
function open_login() {
    open_win('login.html', 'login')
}

/**
 * @function open_register
 * @description 打开注册窗口
 */
function open_register() {
    open_page('register.html', 'register', '注册')
}

/**
 * @function init_common
 * @description 绑定常见的事件
 */
function init_common() {
    $('._logout').on('click', logout)
    $('._login').on('click', open_login)
    $('._to_index').on('click', back_to_index)
    $('._to_cat').on('click', back_to_cat)
    $('._to_cart').on('click', back_to_cart)
    bind_once('._to_user', 'click', back_to_user)
    $('._register').on('click', open_register)
    init_goods()
    init_back()
    $('._list_style').on('click', switch_list_style)
    init_tab()
    init_fold()
    init_back_key()
    init_phone()
    init_ad()
    init_go_to_cart()
    init_checkbox()
    init_page()
    init_compare()
    init_end_time()
    init_switch()
}

/**
 * @function init_compare
 * @description 绑定在线客服按钮
 */
function init_compare() {
    bind_once('._compare', 'click', function() {
       if(typeof($(this).attr('goods_id'))!='object'){
            var goods_image = new Array()
            $('#swiper_container_1').find('img').each(function() {
                goods_image.push($(this).attr('src'))
            })
           
       save_goods_compare({
                goods_id : $(this).attr('goods_id'),
                goods_name : $('#goods_name_label').html(),
                goods_price : $('#goods_price_label').html(),
                goods_image : goods_image,
            })
            }
        open_compare()
         
    })
}
function open_compare(){
    open_page('goods_compare.html', 'goods_compare', '对比')
}

/**
 * function save_goods_compare
 * @description 存储商品比较记录
 * @param {object} data
 */
function save_goods_compare(data) {
    if (!get_val(KEY_GOODS_COMPARE)) {
        var goods_compare = new Object()
    } else {    
        var goods_compare = JSON.parse(get_val(KEY_GOODS_COMPARE))
        if (goods_compare.length > 1000) {
            open_toast('浏览记录超过1000条，无法添加更多记录')
        }
    }
    if ( typeof (goods_compare[data.goods_id]) != 'undefined') {
        data['visit_count'] = goods_compare[data.goods_id]['visit_count'] + 1
    } else {
        data['visit_count'] = 1
    }
    data['last_visit'] = _.now()
    goods_compare[data.goods_id] = data
    set_val(KEY_GOODS_COMPARE, JSON.stringify(goods_compare))
}

/**
 * function clear_goods_compare
 * @description 删除所有比较记录
 */
function clear_goods_compare() {
    appcan.locStorage.remove(KEY_GOODS_COMPARE);
}

/**
 * function delete_goods_compare
 * @description 删除商品比较记录
 * @param {string} goods_id 商品ID
 */
function delete_goods_compare(goods_id) {
    if (get_val(KEY_GOODS_COMPARE)) {
        var goods_compare = JSON.parse(get_val(KEY_GOODS_COMPARE));
        if ( typeof (goods_compare[goods_id]) != 'undefined') {
            delete goods_compare[goods_id];
        }
        set_val(KEY_GOODS_COMPARE, JSON.stringify(goods_compare));
    }
}

/**
 * function get_goods_compare
 * @description 查找商品比较记录
 * @param {object} filter
 * @returns {object} tmp_result 商品历史比较记录
 */
function get_goods_compare(filter) {
    if (_.isUndefined(filter)) {
        filter = new Object();
    }
    if (_.isUndefined(filter.page)) {
        var page = 1;
    } else {
        var page = filter.page;
    }
    if (_.isUndefined(filter.size)) {
        var size = 10;
    } else {
        var size = filter.size;
    }
    if (_.isUndefined(filter.sort_by)) {
        var sort_by = 'last_visit';
    } else {
        var sort_by = filter.sort_by;
    }
    if (_.isUndefined(filter.order)) {
        var order = 'DESC';
    } else {

        var order = filter.order.toUpperCase();
    }

    if (!get_val(KEY_GOODS_COMPARE)) {
        var goods_compare = new Object();
    } else {
        var goods_compare = JSON.parse(get_val(KEY_GOODS_COMPARE));
    }

    var start = (page - 1) * size;
    var end = start + size;
    var tmp_result = _.sortBy(goods_compare, function(data) {
        if (order == 'DESC') {
            return -data[sort_by]
        } else {
            return data[sort_by]
        }
    }).slice(start, end)

    for (i in tmp_result) {
        var commonTime = new Date(tmp_result[i]['last_visit']).toLocaleString()
        tmp_result[i]['formatted_last_visit'] = commonTime
    }
    return tmp_result;
}

/**
 * @function init_page
 * @description 初始化窗口打开函数
 */
function init_page() {
    bind_once('._page', 'click', function() {
        var page_type = $(this).attr("page_type") || 'popover'
        var page_file = $(this).attr("page_file")
        var page_title = $(this).attr("page_title")
        if (!page_file) {
            console.log('empty pagefile')
            return false
        }
        if (page_type === 'popover' && !page_title) {
            console.log('empty page_title')
            return false
        }
        var page_name = $(this).attr("page_name") || page_file
        var page_param = $(this).attr("page_param")
        if (page_param) {
            var page_param = page_param.split('|')
            for (var i in page_param) {
                var tmp = page_param[i]
                if ($(this).attr(tmp)) {
                    set_val(tmp, $(this).attr(tmp))
                }
            }
        }
        if (page_type === 'window') {
            open_win(page_file + '.html', page_name)
        } else if (page_type === 'popover') {
            open_page(page_file + '.html', page_name, page_title)
        }
    })
}

/**
 * @function
 * @name init_checkbox
 * @description 初始化选择框
 */
function init_checkbox() {
    $('._checkbox').each(function() {
        var checked = $(this).prop('checked')
        if ( typeof checked == 'undefined') {
            checked = $(this).attr('checked') == 'true' ? true : false
        }
        $(this).prop('checked', checked)
        bind_once(this, 'click', function() {
            //如果为禁用状态，则直接返回
            if ($(this).attr('disabled') == 'true') {
                return false
            }
            var checked = $(this).prop('checked')
            if ( typeof checked == 'undefined') {
                checked = $(this).attr('checked') == 'true' ? true : false
            }
            var id = $(this).attr('id')
            var value = $(this).attr('value')
            var parent_id = $(this).attr('parent_id')
            var radio = $(this).attr('radio')
            var name = $(this).attr('name')
            var cancel = $(this).attr('cancel')

            //如果为选定状态，且不可取消选择，则直接返回
            if (checked == true && typeof cancel != 'undefined' && cancel == 'false') {
                return false
            }

            checked = !checked
            $(this).prop('checked', checked)
            $(this).attr('checked', checked.toString())
            $(this).toggleClass('checked', checked)

            if ( typeof parent_id != 'undefined' && parent_id != '' && parent_id != null) {
                check_parent_box(parent_id)
            }

            if ( typeof id != 'undefined') {
                check_child_box(checked, id)
            }

            if (checked == true && typeof radio != 'undefined' && radio == 'true' && typeof name != 'undefined' && name != null) {
                uncheck_radio_box(value, name)
            }
        })
    })

    $('._checkbox_label').each(function() {
        var checkbox_id = $(this).attr('for')
        if ( typeof checkbox_id != 'undefined' && checkbox_id != null) {
            $(this).on('click', function() {
                $('#' + checkbox_id).trigger('click')
            })
        }
    })
}

/**
 * @function
 * @name uncheck_radio_box
 * @description 检查单选选择框的选择状态
 *              即把相同名称的单选选择框取消选中状态
 * @param {string} value 选择框的值
 * @param {string} name 选择框的名称
 */
function uncheck_radio_box(value, name) {
    name = name.split('|')
    $('._checkbox').each(function() {
        var cur_value = $(this).attr('value')
        var cur_radio = $(this).attr('radio')
        var cur_name = $(this).attr('name').split('|')
        var disabled = $(this).attr('disabled')

        if (!( typeof disabled != 'undefined' && disabled == 'true')) {
            if (cur_value != value && typeof cur_radio != 'undefined' && cur_radio == 'true' && typeof cur_name != 'undefined' && _.intersection(cur_name, name).length > 0) {
                $(this).prop('checked', false)
                $(this).attr('checked', 'false')
                $(this).removeClass('checked')
            }
        }
    })
}

/**
 * @function
 * @name check_child_box
 * @description 检测子级选择框的状态
 * @param {boolean} 该选择框的选中状态
 * @param {string} 该选择框的ID
 * @param {string} 该选择框的父级的ID
 */
function check_child_box(checked, id) {
    $('._checkbox').each(function() {
        var cur_id = $(this).attr('id')
        var cur_parent_id = $(this).attr('parent_id')

        if (cur_parent_id == id) {
            //如果为禁用状态，直接跳过
            if ($(this).attr('disabled') == 'true') {
                return false
            }
            $(this).prop('checked', checked)
            $(this).toggleClass('checked', checked)
            return check_child_box(checked, cur_id, cur_parent_id)
        }
    })
}

/**
 * @function
 * @name check_parent_box
 * @description 检测父级选择框的状态
 * @param {string} 该选择框的ID
 * @returns {undefined}
 */
function check_parent_box(id) {
    if ($('#' + id).attr('disabled') == 'true') {
        return false
    }
    var prev_checked = $('#' + id).prop('checked')
    var checked = true
    $('._checkbox').each(function() {
        if ($(this).attr('parent_id') == id) {
            checked = checked && $(this).prop('checked')
        }
    })
    $('#' + id).prop('checked', checked)
    $('#' + id).toggleClass('checked', checked)
    if (prev_checked != checked) {
        var parent_id = $('#' + id).attr('parent_id')
        if ( typeof parent_id != 'undefined' && parent_id != '' && parent_id != null) {
            return check_parent_box(parent_id)
        }

        return true
    } else {
        return false
    }
}

/**
 * @function init_go_to_cart
 * @description 初始化打开购物车事件
 */
function init_go_to_cart() {
    bind_once('#go_to_cart', 'click', open_cart, 'left_close_header')
}

/**
 * @function init_back
 * @description 初始化返回事件
 */
function init_back() {
    bind_once('._back', 'click', back)
}
/**
 * @function init_compare
 * @description 绑定对比
 */
function init_chat() {
    bind_once('._chat', 'click', function() {
        var chat_attr = $(this).attr('chat_attr').split('|')
        var data = new Object()
        for (i in chat_attr) {
            var attr_name = chat_attr[i]
            var attr_value = $(this).attr(attr_name)
            data[attr_name] = attr_value
        }
        open_chat(data)
    })
}
/**
 * @function init_ad
 * @description 初始化广告
 */
function init_ad() {
    $('.ad').off('click').on('click', function() {
        var ad_link = $(this).attr('ad_link')
        open_ad(ad_link)
    })
}

/**
 * @function open_ad
 * @description 打开广告链接
 * @param {string} ad_link 链接内容
 */
function open_ad(ad_link) {
    //商品详情
    if (ad_link.indexOf('goods/') === 0) {
        var arr = ad_link.split('/')
        if (arr.length < 2) {
            return false
        }
        var goods_id = arr[1]
        open_goods(goods_id)
    }
    //商品分类
    else if (ad_link.indexOf('goods_cat/') === 0) {
        var arr = ad_link.split('/')
        if (arr.length < 2) {
            return false
        }
        var cat_id = arr[1]
        open_goods_list({
            cat_id : cat_id
        })
    }
    //文章详情
    else if (ad_link.indexOf('article/') === 0) {
        var arr = ad_link.split('/')
        if (arr.length < 2) {
            return false
        }
        var article_id = arr[1]
        open_article(article_id)
    }
    //普通链接
    else if (ad_link.indexOf('http://') === 0) {
        var start = 7
        var link = ad_link.substring(start)
        if (link.length <= 0) {
            return false
        }
        open_browser(ad_link)
    }
    //加密链接
    else if (ad_link.indexOf('https://') === 0) {
        var start = 8
        var link = ad_link.substring(start)
        if (link.length <= 0) {
            return false
        }
        open_browser(ad_link)
    }
    //内部链接
    else if (ad_link.indexOf('appurl/') === 0) {
        var arr = ad_link.split('appurl/')
        if (arr.length < 2) {
            return false
        }
        var app_url = arr[1]
        open_app_url(app_url)
    }
}

/**
 * @function init_bounce
 * @description 初始化下拉刷新和上拉加载
 */
function init_bounce() {
    if (page_type != 'popover') {
        return false
    }
    var option = {
        type : ['up'],
    }
    if ( typeof page_param != 'undefined' && typeof page_param['page'] != 'undefined') {
        option['type'] = ['up', 'down']
    }
    if ( typeof $.bounce != 'undefined') {
        $.bounce.init(option)
    }
}

/**
 * @function init_tab
 * @description 初始化折叠隐藏函数
 */
function init_fold() {
    $('._fold').on('click', function() {
        $(this).toggleClass('expand')
        $('#' + $(this).attr('fold_key')).toggleClass('uhide')
    })
}

/**
 * @function init_tab
 * @description 初始化标签
 */
function init_tab() {
    $('._tab').on('click', function() {
        change_tab($(this).attr('tab_key'))
    })
}

/**
 * @function change_tab
 * @description 切换标签
 * @param {string} 标签唯一标识
 */
function change_tab(tab_key) {
    $('._tab').removeClass('selected')
    $('#tab_' + tab_key).addClass('selected')
    $('._tab_content').addClass('uhide')
    $('#tab_content_' + tab_key).removeClass('uhide')
}

/**
 * @function back_to_index
 * @description 返回到root窗口
 *              然后打开首页
 */
function back_to_index() {
    eval_script('root', 'setTimeout(function(){open_con(0)},DUR_ANIMATION)')
    back({
        target : 'root'
    })
}

/**
 * @function back_to_cat
 * @description 返回到root窗口
 *              然后打开分类
 */
function back_to_cat() {
    eval_script('root', 'setTimeout(function(){open_con(1)},DUR_ANIMATION)')
    back({
        target : 'root'
    })
}
/*
 * @function back_to_brands
 * @description 返回到brand窗口
 *              然后打开品牌
 */
function back_to_brand() {
    eval_script('root', 'setTimeout(function(){open_con(1)},DUR_ANIMATION)')
    back({
        target : 'root'
    })
}
/**
 * @function back_to_cart
 * @description 返回到root窗口
 *              然后打开购物车
 */
function back_to_cart() {
    eval_script('root', 'setTimeout(function(){open_con(3)},DUR_ANIMATION)')
    back({
        target : 'root'
    })
}

/**
 * @function back_to_user
 * @description 返回到root窗口
 *              然后打开用户中心
 * @param {string} script 打开用户中心后执行的脚本
 */
function back_to_user(script) {
    eval_script('root', 'setTimeout(function(){open_con(4)},DUR_ANIMATION)')
    eval_script('root', 'setTimeout(function(){' + script + '},DUR_ANIMATION+DUR_SECOND/2)')
    back({
        target : 'root'
    })
}

/**
 * @function logout
 * @description 登出
 */
function logout() {
    ajax({
        data : {
            script_name : 'user',
            act : 'logout'
        },
        success : cb_logout
    })
}

/**
 * @function cb_logout
 * @description 登出回调函数
 * @param {string} data 服务器端返回的数据
 */
function cb_logout(data) {
    var json = parse_result(data)
    if (json.logout_success == 1) {
        del_val('user_name')
        del_val(KEY_LOCATION)
        //del_val(KEY_REGION)
        del_val(KEY_GOODS_HISTORY)
        del_val(KEY_SEARCH_HISTORY)
        open_toast(json.message)
        publish(CHANNEL_LOGOUT_SUCCESS)
    }
}

/**
 * @function open_order_list
 * @description 打开订单列表
 * @param {string} composite_status 订单综合状态
 */
function open_order_list(composite_status) {
    if (arguments.length === 1 && $.isPlainObject(composite_status)) {
        composite_status = composite_status.composite_status
    }
    set_val('composite_status', composite_status)
    open_page('order_list.html', 'order_list', '订单列表')
}

/**
 * @function open_goods_history
 * @description 打开商品浏览历史
 */
function open_goods_history() {
    open_page('goods_history.html', 'goods_history', '浏览历史')
}

/**
 * @function open_user_info
 * @description 打开个人信息窗口
 */
function open_user_info() {
    open_page('user_info.html', 'user_info', '个人信息')
}

/**
 * @function open_virtual_card
 * @description 打开储值卡列表窗口
 */
function open_virtual_card() {
    open_page('user_vc_login.html', 'user_vc_login', '储值卡')
}

/**
 * @function open_my_comment
 * @description 打开评价、晒单列表
 * @param {string} order_id 订单ID
 * @param {string} comment_status 评论状态
 */
function open_my_comment(order_id, comment_status) {
    if (arguments.length === 1 && $.isPlainObject(order_id)) {
        comment_status = order_id.comment_status
        order_id = order_id.order_id
    }
    set_val('comment_status', comment_status)
    set_val('order_id', order_id)
    open_page('user_my_comment.html', 'user_my_comment', '商品评价/晒单')
}

/**
 * @function open_collection_list
 * @description 打开商品收藏列表
 */
function open_collection_list() {
    open_page('user_collection_list.html', 'user_collection_list', '收藏商品')
}

/**
 * @function open_follow_shop
 * @description 打开店铺关注列表
 */
function open_follow_shop() {
    open_page('user_follow_shop.html', 'user_follow_shop', '关注的店铺')
}

/**
 * @function open_user_bonus
 * @description 打开用户优惠券列表
 */
function open_user_bonus() {
    open_page('user_bonus.html', 'user_bonus', '优惠券')
}

function open_about() {
    open_page('user_about.html', 'about', '关于我们')
}


/**
 * @function open_user_account
 * @description 打开用户余额窗口
 */
function open_user_account() {
    open_page('user_account.html', 'user_account', '会员余额')
}

/**
 * @function open_user_message_list
 * @description 打开用户留言窗口
 */
function open_user_message_list() {
    open_page({
        win_name : 'user_message_list',
        win_url : 'user_message_list.html',
        win_title : '我的留言',
        header : 'user_message_header',
    })
}

/**
 * @function open_booking_list
 * @description 打开缺货登记窗口
 */
function open_booking_list() {
    open_page('user_booking_list.html', 'user_booking_list', '缺货登记')
}

/**
 * @function open_consignee_list
 * @description 打开收货人列表窗口
 * @param {string} script_name 脚本名称 ，即page_param['script_name]
 */
function open_consignee_list(script_name) {
    set_val('script_name', script_name)
    open_page('consignee_list.html', 'consignee_list', '收货地址')
}

/**
 * @function open_consignee
 * @description 打开修改收货地址窗口
 *              由于被多个功能共用（结算时修改收货地址，个人中心修改收货地址，订单详情修改收货地址）
 *              需要多个参数
 * @param {string} script_name 脚本名称 ，即page_param['script_name]
 * @param {string} address_id 地址唯一标识
 * @param {string} order_id 订单ID
 * @param {string} act act参数
 * @param {string} confirm_act confirm_act参数
 */
function open_consignee(script_name, address_id, order_id, act, confirm_act, title) {
    if (arguments.length === 1 && $.isPlainObject(script_name)) {
        address_id = script_name.address_id
        order_id = script_name.order_id
        act = script_name.act
        confirm_act = script_name.confirm_act
        title = script_name.title
        script_name = script_name.script_name
    }
    if ( typeof title == 'undefined') {
        title = '修改地址'
    }
    set_val('address_id', address_id)
    set_val('script_name', script_name)
    set_val('order_id', order_id)
    set_val('act', act)
    set_val('confirm_act', confirm_act)
    open_page('consignee.html', 'consignee', title)
}

/**
 * @function open_checkout
 * @description 打开结算窗口
 * @param {array} selected_goods
 * @param {array} goods_number
 */
function open_checkout(selected_goods, goods_number) {
    if (_.isEmpty(selected_goods)) {
        open_toast('请选择商品')
        return false
    }
    set_val('selected_goods', JSON.stringify(selected_goods))
    set_val('goods_number', JSON.stringify(goods_number))
    open_page('checkout.html', 'checkout', '结算')
}

/**
 * @function open_article_list
 * @description 打开文章列表
 * @param {string} cat_id 文章分类ID
 * @param {string} cat_name 文章分类名称
 */
function open_article_list(cat_id, cat_name) {
    set_val('cat_id', cat_id)
    if ( typeof cat_name == 'undefined') {
        cat_name = '文章列表'
    }
    open_page('article_list.html', 'article_list', cat_name)
}

/**
 * @function open_find_password
 * @description 打开找回密码窗口
 */
function open_find_password() {
    open_page('find_password.html', 'find_password', '找回密码')
}

/**
 * @function open_pay
 * @description 打开支付窗口
 * @param {string} pay_online 支付代码
 */
function open_pay(pay_online) {
    set_val('pay_online', pay_online)
    if (pay_online.indexOf('weixin/') === 0 || pay_online.indexOf('alipay/') === 0) {
        open_win({
            win_url : 'pay.html',
            win_name : 'pay',
            win_type : WIN_OATH,
        })
    } else {
        open_page({
            win_url : 'pay.html',
            win_name : 'pay',
            win_title : '支付',
            win_type : WIN_OATH,
        })
    }
}

/**
 * @function open_order
 * @description 打开订单详情
 */
function open_order(order_id) {
    set_val('order_id', order_id)
    open_page('order_detail.html', 'order_detail', '订单详情')
}

/**
 * @function open_deposit
 * @description 打开充值窗口
 */
function open_deposit() {
    open_page('user_account_deposit.html', 'user_account_deposit', '充值')
}

/**
 * @function open_withdraw
 * @description 打开提现窗口
 */
function open_withdraw() {
    open_page('user_account_raply.html', 'user_account_raply', '提现')
}

/**
 * @function open_stores
 * @description 打开店铺列表
 * @param {string} keywords 关键词
 */
function open_stores(keywords) {
    set_val('keywords', keywords)
    open_page('stores.html', 'stores', '搜索店铺：' + keywords)
}

/**
 * @function init_empty_image
 * @description 切换商品列表显示样式
 */
function switch_list_style() {
    $('#goods_list_container').toggleClass('goodlist-b goodlist-s')
    $('#style_button').toggleClass('top-r-glist1 top-r-glist2')
}

/**
 * @function init_empty_image
 * @description 处理错误图片
 */
function init_empty_image() {
    $('img').on('error', function() {
        $(this).attr('src', 'img/empty_image.png')
    })
}

/**
 * @function open_promote_list
 * @description 打开促销列表
 */
function open_promote_list() {
    set_val('is_promote', '1')
    open_page('goods_list.html', 'goods_list', '限时抢购')
}

/**
 * @function open_express
 * @description 打开物流信息窗口
 * @param {string} express_number 快递单号
 * @param {string} express_name 快递名称
 * @param {string} order_id 订单ID
 */
function open_express(express_number, express_name, order_id) {
    set_val('express_number', express_number)
    set_val('express_name', express_name)
    set_val('order_id', order_id)
    set_val('url', 'http://m.kuaidi100.com/result.jsp?nu=' + express_number)
    open_win('express.html', 'express')
}

/**
 * @function open_barcode_result
 * @description 打开条形码搜索结果
 * @param {string} code 条形码
 */
function open_barcode_result(code) {
    set_val('barcode', code)
    open_page({
        win_url : 'barcode_result.html',
        win_name : 'barcode_result',
        win_title : '条形码扫描结果',
        header : 'right_scan_header',
    })
}

/**
 * @function receive_message
 * @description 收到一条消息
 * @param {string} data 消息内容
 */
function receive_message(data) {
    var json = JSON.parse(data)
    var message = json.message
    var extras = json.extras
    var type = get_message_type(extras)
    var extra = get_message_extra(extras)
    var title = get_message_title(json)
    save_message({
        message : message,
        type : type,
        extra : extra,
        title : title,
        is_read : 0,
    })
    play_audio(AUD_MESSAGE)
}

/**
 * @function open_notification
 * @description 打开一条通知 并存储
 * @param {string} data 通知内容
 */
function open_notification(data) {
    var json = JSON.parse(data)
    var message = json.content
    var extras = json.extras
    var type = get_message_type(extras)
    var extra = get_message_extra(extras)
    var title = get_message_title(json)
    save_message({
        message : message,
        type : type,
        extra : extra,
        title : title,
        is_read : 1,
    })
    open_extras(type, extra)
}

/**
 * @function receive_notification
 * @description 收到一条通知
 * @param {string} data 通知内容
 */
function receive_notification(data) {
    var json = JSON.parse(data)
    var message = json.content
    var extras = json.extras
    var type = get_message_type(extras)
    var extra = get_message_extra(extras)
    var title = get_message_title(json)
    save_message({
        message : message,
        type : type,
        extra : extra,
        title : title,
        is_read : 0,
    })
}

/**
 * @function delete_message
 * @description 保存消息一条消息
 * @param {object} m_obj 一条消息
 */
function save_message(m_obj) {
    var all_message = get_val(KEY_MESSAGE)
    if (_.isEmpty(all_message)) {
        all_message = new Array()
    } else {
        all_message = JSON.parse(all_message)
    }
    if (all_message.length > 100) {
        open_toast('本地消息超过100条，将无法存储更多消息')
        return false
    }
    var is_exist = false
    for (i in all_message) {
        if (is_same_message(all_message[i], m_obj)) {
            $.extend(all_message[i], m_obj)
            is_exist = true
            break
        }
    }
    m_obj.time = _.now()
    if (!is_exist) {
        all_message.push(m_obj)
    }
    set_val(KEY_MESSAGE, JSON.stringify(all_message))
    init_message_count()
}

/**
 * @function delete_message
 * @description 删除一条消息
 * @param {object} m_obj 一条消息
 */
function delete_message(m_obj) {
    var all_message = get_val(KEY_MESSAGE)

    if (_.isEmpty(all_message)) {
        all_message = new Array()
    } else {
        all_message = JSON.parse(all_message)
    }

    for (i in all_message) {
        if (is_same_message(all_message[i], m_obj)) {
            all_message.splice(i, 1)
            break
        }
    }
    set_val(KEY_MESSAGE, JSON.stringify(all_message))
    init_message_count()
}

/**
 * @function clear_message
 * @description 清空本地消息
 */
function clear_message() {
    set_val(KEY_MESSAGE, JSON.stringify(new Array()))
}

/**
 * @function get_message_type
 * @description 获取一条消息的类型
 * @param {object} extras 消息额外参数
 * @returns {string} type 消息类型
 */
function get_message_type(extras) {
    var type = 'common'
    if ( typeof extras['goods'] != 'undefined') {
        type = 'goods'
    } else if ( typeof extras['article'] != 'undefined') {
        type = 'article'
    } else if ( typeof extras['url'] != 'undefined') {
        type = 'url'
    }
    return type
}

/**
 * @function get_message_title
 * @description 获取一条消息的额外参数 的内容
 * @param {object} extras 消息额外参数
 * @returns {string} extra 消息额外参数的内容
 */
function get_message_extra(extras) {
    var extra = ''
    if ( typeof extras['goods'] != 'undefined') {
        extra = extras['goods']
    } else if ( typeof extras['article'] != 'undefined') {
        extra = extras['article']
    } else if ( typeof extras['url'] != 'undefined') {
        extra = extras['url']
    }
    return extra
}

/**
 * @function get_message_title
 * @description 获取消息标题
 * @param {object} data 一条消息
 * @returns {string} title 标题
 */
function get_message_title(data) {
    var title = '消息通知'
    if ( typeof data.title != 'undefined' && !_.isEmpty(data.title)) {
        title = data.title
    } else {
        var extras = data.extras
        if ( typeof extras['goods'] != 'undefined') {
            title = '商品推荐'
        } else if ( typeof extras['article'] != 'undefined') {
            title = '文章推荐'
        } else if ( typeof extras['url'] != 'undefined') {
            title = '链接推荐'
        }
    }
    return title
}

/**
 * @function init_message_count
 * @description 更新未读消息数量
 */
function init_message_count() {
    var all_message = get_val(KEY_MESSAGE)
    if (_.isEmpty(all_message)) {
        all_message = new Array()
    } else {
        all_message = JSON.parse(all_message)
    }
    var unread_message_count = 0
    for (i in all_message) {
        if (all_message[i]['is_read'] == 0) {
            unread_message_count++
        }
    }
    publish(CHANNEL_UPDATE_MESSAGE, unread_message_count)
}

/**
 * @function is_same_message
 * @description 判断是否为相同的消息
 * @param {object} m_a 消息a
 * @param {object} m_b 消息b
 * @returns {boolean} 如果相同，返回true，不相同则返回
 */
function is_same_message(m_a, m_b) {
    if (m_a['title'] == m_b['title'] && m_a['message'] == m_b['message'] && m_a['type'] == m_b['type'] && m_a['extra'] == m_b['extra']) {
        return true
    } else {
        return false
    }
}

/**
 * @function get_message
 * @description 获取缓存中的消息
 * @param {string} page 页数
 * @param {string} size 数量
 * @param {string} is_read 是否已读
 * @param {string} sort 排序方式，默认为时间
 * @param {string} order 顺序，默认降序
 * @returns {array} result 消息数组
 */
function get_message(page, size, is_read, sort, order) {
    if ( typeof page == 'undefined') {
        page = 0
    }
    if ( typeof size == 'undefined') {
        size = 0
    }
    if ( typeof is_read == 'undefined') {
        is_read = -1
    }

    if ( typeof sort === 'undefined') {
        sort = 'time'
    }

    if ( typeof order === 'undefined') {
        order = 'desc'
    }
    var all_message = get_val(KEY_MESSAGE)
    if (_.isEmpty(all_message)) {
        all_message = new Array()
    } else {
        all_message = JSON.parse(all_message)
    }

    var tmp_result = _.sortBy(all_message, function(data) {
        if (order.toUpperCase() == 'DESC') {
            return -data[sort]
        } else {
            return data[sort]
        }
    })
    if (size > 0) {
        var start = (page - 1) * size
        var result = tmp_result.slice(start, size)
    } else {
        var result = tmp_result
    }

    for (i in result) {
        result[i]['time'] = new Date(result[i]['time']).toLocaleString()
    }
    return result
}

/**
 * @function open_extras
 * @description 打开相应的窗口
 * @param {string} type 消息类型
 * @param {string} extra 消息内容
 */
function open_extras(type, extra) {
    switch(type) {
    case 'goods':
        open_goods(extra)
        break
    case 'article':
        open_article(extra)
        break
    case 'url':
        open_confirm({
            message : '是否打开链接' + extra,
            callback : function(data) {
                if (data == 1) {
                    open_browser(extra)
                }
            }
        })
        break
    default:
        open_message(ANIM_BOTTOM_TO_TOP, 'right_close_header')
        break
    }
}

/**
 * @function open_message
 * @description 打开消息盒子窗口
 * @param {string} anim_id 动画ID
 * @param {string} header 头部名称
 */
function open_message(anim_id, header) {
    if (arguments.length === 1 && $.isPlainObject(anim_id)) {
        header = anim_id.header
        anim_id = anim_id.anim_id
    }
    open_page({
        win_url : 'message.html',
        win_name : 'message',
        win_title : '消息盒子',
        anim_id : anim_id,
        header : header,
    })
}

/**
 * @function open_guide
 * @description 打开引导窗口
 */
function open_guide() {
    open_win('guide.html', 'guide')
}

/**
 * @function play_audio
 * @description 播放音效
 * @param {string} id 音效id
 */
function play_audio(id) {
    setTimeout(function() {
        uexAudio.play(1)
    }, 100)
    uexAudio.open(AUD_PATH[id])
}

/**
 * @function init_back_key
 * @description 初始化后退键
 */
function init_back_key() {
    if (platform() == 'android') {
        if (page_type == 'window') {
            uexWindow.setReportKey(0, 1)
            uexWindow.setReportKey(1, 1)
            uexWindow.onKeyPressed = function(keycode) {
                switch(keycode) {
                case 0:
                    back()
                    break
                }
            }
        } else if (page_type == 'popover') {
            uexWindow.setReportKey(0, 1)
            uexWindow.setReportKey(1, 1)
        }
    }
}

/**
 * @function init_goods
 * @description 初始化商品相关事件
 *              如加入购物车，打开商品，加减数量
 */
function init_goods() {
    $('.goods').each(function() {
        var goods_id = $(this).attr('goods_id')
        if ($(this).find('.goods-img,.goods-name').length > 0) {
            $(this).find('.goods-img,.goods-name').off('click').on('click', function() {
                open_goods(goods_id)
            })
        } else {
            $(this).off('click').on('click', function() {
                open_goods(goods_id)
            })
        }
        $(this).find('.add_to_cart').off('click').on('click', function() {
            var goods_number = 1
            if ($('#goods_number_' + goods_id).length > 0) {
                goods_number = $('#goods_number_' + goods_id).val()
            }
            add_to_cart(goods_id, goods_number)
        })
        $(this).find('img').on('error', function() {
            $(this).attr('src', 'img/empty_image.png')
        })
        $(this).find('.settime').off('click').on('click', function() {
            open_goods(goods_id)
        })

        $(this).find('.minus').off('click').on('click', function() {
            if (!$(this).hasClass('disabled')) {
                minus_goods_number(goods_id)
            }
        })

        $(this).find('.plus').off('click').on('click', function() {
            if (!$(this).hasClass('disabled')) {
                plus_goods_number(goods_id)
            }
        })
    })
}

/**
 * @function minus_goods_number
 * @description 商品列表减少商品数量
 * @param {string} goods_id 商品ID
 */
function minus_goods_number(goods_id) {
    var current_goods_number = $('#goods_number_' + goods_id).val()
    if (current_goods_number == '1') {
        return
    }
    if (current_goods_number == '2') {
        $('#minus_' + goods_id).addClass('disabled')
    }

    current_goods_number = parseInt(current_goods_number) - 1

    if ($('#plus_' + goods_id).attr('buy_max') > 0) {
        if (current_goods_number < $('#plus_' + goods_id).attr('buy_max')) {
            $('#plus_' + goods_id).removeClass('disabled')
        } else {
            $('#plus_' + goods_id).addClass('disabled')
        }
    } else {
        $('#plus_' + goods_id).removeClass('disabled')
    }
    $('#goods_number_' + goods_id).val(current_goods_number)
}

/**
 * @function plus_goods_number
 * @description 商品列表增加商品数量
 * @param {string} goods_id 商品ID
 */
function plus_goods_number(goods_id) {
    var current_goods_number = $('#goods_number_' + goods_id).val()
    current_goods_number = parseInt(current_goods_number) + 1
    if ($('#plus_' + goods_id).attr('buy_max') > 0) {
        if (current_goods_number < $('#plus_' + goods_id).attr('buy_max')) {
            $('#plus_' + goods_id).removeClass('disabled')
        } else {
            $('#plus_' + goods_id).addClass('disabled')
        }
    } else {
        $('#plus_' + goods_id).removeClass('disabled')
    }

    if (current_goods_number > 1) {
        $('#minus_' + goods_id).removeClass('disabled')
    }
    $('#goods_number_' + goods_id).val(current_goods_number)
}

/**
 * @function open_debug
 * @description 打开调试窗口
 */
function open_debug() {
    open_page('debug.html', 'debug', '调试模式')
}

/***
 * @function bind_once
 * @description 封装Zepto.on函数，主要为了避免重复绑定同一事件
 *              即绑定一个事件时，会取消所有绑定的相同的事件
 * @param {string} selector 选择器或html
 * @param {string} evt 需要绑定的事件
 * @param {funciton} func 绑定的函数
 * @param {arguments} arg 绑定的函数
 */
function bind_once(selector, evt, func, arg) {
    $(selector).off(evt)
    bind.apply(bind, arguments)
}

/***
 * @function bind
 * @description 绑定事件
 * @param {string} selector 选择器或html
 * @param {string} evt 需要绑定的事件
 * @param {funciton} func 绑定的函数
 * @param {arguments} arg 绑定的函数
 */
function bind(selector, evt, func, arg) {
    if (arguments.length > 3) {
        var args = new Array()
        for (var i = 3; i <= arguments.length; i++) {
            args.push(arguments[i])
        }
    } else {
        var args = new Array()
    }
    $(selector).on(evt, function(e) {
        func.apply(this, args)
        return false
    })
}

/***
 * @function set_region
 * @description 打开自提点列表窗口
 * @param {string} suppid 打开自提点窗口
 */
function open_pickup_info(suppid) {
    set_val('suppid', suppid)
    open_page('pickup_info.html', 'pickup_info', '自提点列表')
}

/**
 * @function set_region
 * @description 保存区域
 * @param {object} region_info 区域信息
 */
function set_region(region_info) {
    var region = get_region()
    $.extend(true, region, region_info)
    set_val(KEY_REGION, JSON.stringify(region))
}

/**
 * @function get_region
 * @description 获取缓存中的区域
 * @returns {object} 区域信息
 */
function get_region() {
    var region = get_val(KEY_REGION)
    if (_.isEmpty(region)) {
        region = {
            province : {
                id : '0',
                name : '未知'
            },
            city : {
                id : '0',
                name : '未知'
            },
            district : {
                id : '0',
                name : '未知'
            },
            xiangcun : {
                id : '0',
                name : '未知'
            },
        }
    } else {
        region = JSON.parse(region)
    }
    return region
}

/**
 * @function set_location
 * @description 存储地理位置
 * @param {object} location_info 地理位置信息
 */
function set_location(location_info) {
    location_info.time = _.now()
    var location = get_location()
    $.extend(true, location, location_info)
    set_val(KEY_LOCATION, JSON.stringify(location))
}

/**
 * @function get_location
 * @description 获取缓存中的地理位置
 */
function get_location() {
    var location = get_val(KEY_LOCATION)
    if (_.isEmpty(location)) {
        return {
            time : 0,
            lat : 0,
            log : 0,
            province : {
                name : '未知',
                id : '0'
            },
            city : {
                name : '未知',
                id : '0'
            },
            district : {
                name : '未知',
                id : '0'
            },
            xiangcun : {
                name : '未知',
                id : '0'
            },
        }
    } else {
        return JSON.parse(location)
    }
}

/**
 * @function is_need_reloc
 * @description 判断是否需要重新定位
 *              根据DUR_CHECK_GPS_LOCATION变量判断
 * @returns {boolean} true|false  是否需要重新定位
 */
function is_need_reloc() {
    var location = get_val(KEY_LOCATION)
    if (_.isEmpty(location)) {
        return true
    } else {
        location = JSON.parse(location)
        var last_time = location.time
        var now = _.now()
        if (now - last_time >= DUR_CHECK_GPS_LOCATION) {
            return true
        } else {
            return false
        }
    }
}

/**
 * @function get_address
 * @description 获取GPS定位水
 */
function get_address() {
    if (platform() === 'simulator') {
        set_location({
            lat : 39.9411100000,
            log : 119.5999300000,
        })
        cb_get_address('1', '1', '{"addressComponent":{"province":"河北省","city":"秦皇岛市","district":"海港区","street":"建设大街254号"}}')
        return true
    }
    appcan.get_addr_t = setTimeout(function() {
        publish(CHANNEL_GET_ADDRESS_FAIL)
    }, DUR_GET_ADDRESS)
    uexLocation.closeLocation()
    uexLocation.onChange = function(lat, log) {
        uexLocation.closeLocation()
        set_location({
            lat : lat,
            log : log,
        })
        uexLocation.cbGetAddress = cb_get_address
        uexLocation.getAddress(lat, log, 1)
    }
    uexLocation.cbOpenLocation = function(opId, dataType, data) {
        if (data == 1) {
            clearTimeout(appcan.get_addr_t)
            publish(CHANNEL_GET_ADDRESS_FAIL)
        }
    }
    uexLocation.openLocation()
}

/**
 * @function cb_get_address
 * @description 获取详细地址回调函数
 * @param {string} opCode
 * @param {string} dataType
 * @param {string} data
 */
function cb_get_address(opCode, dataType, data) {
    uexLocation.closeLocation()
    try {
        data = JSON.parse(data)
    } catch(e) {
        clearTimeout(appcan.get_addr_t)
        publish(CHANNEL_GET_ADDRESS_FAIL)
        return false
    }
    var province_name = data.addressComponent.province
    var city_name = data.addressComponent.city
    var district_name = data.addressComponent.district
    var xiangcun_name = data.addressComponent.street

    var location = {
        province : {
            name : province_name
        },
        city : {
            name : city_name
        },
        district : {
            name : district_name
        },
        xiangcun : {
            name : xiangcun_name
        },
    }
    set_location(location)
    get_region_id()
}

/**
 * @function get_region_id
 * @description 获取地区ID
 */
function get_region_id() {
    var data = {
        script_name : 'root',
        act : 'get_region_id',
    }
    var location = get_location()
    var province = location.province
    var city = location.city
    var district = location.district
    var xiangcun = location.xiangcun

    if ( typeof district.name != 'undefined') {
        data.province_name = province.name
    }
    if ( typeof city.name != 'undefined') {
        data.city_name = city.name
    }
    if ( typeof district.name != 'undefined') {
        data.district_name = district.name
    }
    if ( typeof xiangcun.name != 'undefined') {
        data.xiangcun_name = xiangcun.name
    }
    ajax({
        data : data,
        success : cb_get_region_id,
    })
}

/**
 * @function cb_get_region_id
 * @description 获取地区ID回调函数
 * @param {string} data 服务器端返回的数据
 */
function cb_get_region_id(data) {
    try {
        data = JSON.parse(data)
    } catch(e) {
        clearTimeout(appcan.get_addr_t)
        publish(CHANNEL_GET_ADDRESS_FAIL)
        return false
    }

    var province_id = data.province_id
    var province_name = data.province
    var city_id = data.city_id
    var city_name = data.city
    var district_id = data.district_id
    var district_name = data.district
    var xiangcun_id = data.xiangcun_id
    var xiangcun_name = data.xiangcun_name

    var location = {
        province : {
            s_name : province_name,
            id : province_id,
        },
        city : {
            s_name : city_name,
            id : city_id,
        },
        district : {
            s_name : district_name,
            id : district_id,
        },
        xiangcun : {
            s_name : xiangcun_name,
            id : xiangcun_id,
        },
    }
    set_location(location)
    clearTimeout(appcan.get_addr_t)
    publish(CHANNEL_GET_ADDRESS)
}

/**
 * @function init_phone
 * @description 初始化电话
 */
function init_phone() {
    $('._phone').on('click', function() {
        var phone = $(this).attr('phone')
        open_phone(phone)
    })
}

/**
 * @function init_menu
 * @description 初始化菜单
 */
function init_menu() {
    bind_once('._menu', 'click', function() {
        var type = $(this).attr('menu_type')
        var link = $(this).attr('menu_link')
        var name = $(this).attr('menu_name')
        open_menu(type, link, name)
    })
}

/**
 * @function open_menu
 * @description 打开首页菜单
 * @param {string} type 窗口类型
 *                      对应后台管理的窗口类型
 * @param {string} link 窗口连接
 * @param {string} name 窗口名称
 */
function open_menu(type, link, name) {
    switch(type) {
    case 'link':
        open_browser(link, name)
        break
    case 'page':
        open_menu_page(link, name)
        break
    case 'goods':
        open_goods(link)
        break
    case 'goods_cat':
        open_goods_list({
            cat_id : link
        })
        break
    case 'supplier':
        open_supplier(link, name)
        break
    case 'article':
        open_article(link)
        break
    case 'article_cat':
        open_article_cat(link, name)
        break
    case 'article_list':
        open_article_list(link, name)
        break
    case 'phone':
        open_phone(link)
        break
    case 'brand':
        open_brand(link, name)
        break
    case 'appurl':
        open_app_url(link, name)
        break
    case 'custom':
        open_custom_menu(link, name)
        break
    }
}

/**
 * @function open_custom_menu
 * @description 自定义菜单选项
 *              对应后台管理中的自定义窗口
 *              需要自己处理
 * @param {string} link 窗口连接
 * @param {string} name 窗口名称
 */
function open_custom_menu(link, name) {
    /* put your code here */
}

/**
 * @function open_menu_page
 * @description 打开首页菜单所对应的窗口
 * @param {string} link 窗口连接
 * @param {string} name 窗口名称
 */
function open_menu_page(link, name) {
    switch(link) {
    case 'promote_list':
        open_promote_list()
        break
    case 'exclusive':
        open_exclusive(name)
        break
    default:
        open_page(link + '.html', link, name)
        break
    }
}

/**
 * @function open_exclusive
 * @description 打开手机专享窗口
 * @param {string} name 窗口名称
 */
function open_exclusive(name) {
    set_val('exclusive', 1)
    open_page('goods_list.html', 'exclusive', name)
}

/**
 * @function open_phone
 * @description 拨打电话号码
 * @param {string} phone 电话号码
 */
function open_phone(phone) {
    open_confirm({
        message : '是否拨打电话：' + phone,
        callback : function(data) {
            if (data == 1) {
                uexCall.call(phone)
            }
        }
    })
}

/**
 * @function open_article_cat
 * @description 打开文章分类窗口
 * @param {string} cat_id 文章分类ID
 * @param {string} title 窗口标题
 */
function open_article_cat(cat_id, title) {
    if ( typeof title == 'undefined') {
        title = '文章分类'
    }
    set_val('cat_id', cat_id)
    open_page('article_cat.html', 'article_cat', title)
}

/**
 * @function open_activity
 * @description 打开活动窗口
 */
function open_activity() {
    open_page('activity.html', 'activity', '优惠活动')
}

/**
 * @function get_script_async
 * @description 以异步方式加载文件
 * @param {string} file_path 文件路径
 * @param {function} callback 回调函数
 */
function get_script_async(file_path, callback) {
    var script = document.createElement('script')
    script.src = file_path
    script.type = 'text/javascript'
    if (callback) {
        script.onload = callback
        script.onreadystatechange = callback
    }
    document.getElementsByTagName("head")[0].appendChild(script)
}

/**
 * @function get_script_sync
 * @description 以同步方式加载文件
 * @param {string} file_path 文件路径
 */
function get_script_sync(file_path) {
    var req = new XMLHttpRequest()
    req.open("GET", file_path, false)
    req.send(null)
    var headElement = document.getElementsByTagName("head")[0]
    var newScriptElement = document.createElement("script")
    newScriptElement.type = "text/javascript"
    newScriptElement.text = req.responseText
    headElement.appendChild(newScriptElement)
}

/**
 * @function get_base_uri
 * @description 获取当前页面的路径
 * @returns {string} base_uri 当前页面的路径
 */
function get_base_uri() {
    var uri = window.location.href
    var file_name = uri.split('/').pop()
    var base_uri = uri.replace(file_name, '')
    return base_uri
}

/***
 * @function md5
 * @description md5加密
 * @param {string} str 需要加密的字符串
 * @returns {string} md5 加密后的字符串
 */
function md5(str) {
    var xl;

    var rotateLeft = function(lValue, iShiftBits) {
        return (lValue << iShiftBits) | (lValue >>> (32 - iShiftBits));
    };

    var addUnsigned = function(lX, lY) {
        var lX4,
            lY4,
            lX8,
            lY8,
            lResult;
        lX8 = (lX & 0x80000000);
        lY8 = (lY & 0x80000000);
        lX4 = (lX & 0x40000000);
        lY4 = (lY & 0x40000000);
        lResult = (lX & 0x3FFFFFFF) + (lY & 0x3FFFFFFF);
        if (lX4 & lY4) {
            return (lResult ^ 0x80000000 ^ lX8 ^ lY8);
        }
        if (lX4 | lY4) {
            if (lResult & 0x40000000) {
                return (lResult ^ 0xC0000000 ^ lX8 ^ lY8);
            } else {
                return (lResult ^ 0x40000000 ^ lX8 ^ lY8);
            }
        } else {
            return (lResult ^ lX8 ^ lY8);
        }
    };

    var _F = function(x, y, z) {
        return (x & y) | ((~x) & z);
    };
    var _G = function(x, y, z) {
        return (x & z) | (y & (~z));
    };
    var _H = function(x, y, z) {
        return (x ^ y ^ z);
    };
    var _I = function(x, y, z) {
        return (y ^ (x | (~z)));
    };

    var _FF = function(a, b, c, d, x, s, ac) {
        a = addUnsigned(a, addUnsigned(addUnsigned(_F(b, c, d), x), ac));
        return addUnsigned(rotateLeft(a, s), b);
    };

    var _GG = function(a, b, c, d, x, s, ac) {
        a = addUnsigned(a, addUnsigned(addUnsigned(_G(b, c, d), x), ac));
        return addUnsigned(rotateLeft(a, s), b);
    };

    var _HH = function(a, b, c, d, x, s, ac) {
        a = addUnsigned(a, addUnsigned(addUnsigned(_H(b, c, d), x), ac));
        return addUnsigned(rotateLeft(a, s), b);
    };

    var _II = function(a, b, c, d, x, s, ac) {
        a = addUnsigned(a, addUnsigned(addUnsigned(_I(b, c, d), x), ac));
        return addUnsigned(rotateLeft(a, s), b);
    };

    var convertToWordArray = function(str) {
        var lWordCount;
        var lMessageLength = str.length;
        var lNumberOfWords_temp1 = lMessageLength + 8;
        var lNumberOfWords_temp2 = (lNumberOfWords_temp1 - (lNumberOfWords_temp1 % 64)) / 64;
        var lNumberOfWords = (lNumberOfWords_temp2 + 1) * 16;
        var lWordArray = new Array(lNumberOfWords - 1);
        var lBytePosition = 0;
        var lByteCount = 0;
        while (lByteCount < lMessageLength) {
            lWordCount = (lByteCount - (lByteCount % 4)) / 4;
            lBytePosition = (lByteCount % 4) * 8;
            lWordArray[lWordCount] = (lWordArray[lWordCount] | (str.charCodeAt(lByteCount) << lBytePosition));
            lByteCount++;
        }
        lWordCount = (lByteCount - (lByteCount % 4)) / 4;
        lBytePosition = (lByteCount % 4) * 8;
        lWordArray[lWordCount] = lWordArray[lWordCount] | (0x80 << lBytePosition);
        lWordArray[lNumberOfWords - 2] = lMessageLength << 3;
        lWordArray[lNumberOfWords - 1] = lMessageLength >>> 29;
        return lWordArray;
    };

    var wordToHex = function(lValue) {
        var wordToHexValue = '',
            wordToHexValue_temp = '',
            lByte,
            lCount;
        for ( lCount = 0; lCount <= 3; lCount++) {
            lByte = (lValue >>> (lCount * 8)) & 255;
            wordToHexValue_temp = '0' + lByte.toString(16);
            wordToHexValue = wordToHexValue + wordToHexValue_temp.substring(wordToHexValue_temp.length - 2, wordToHexValue_temp.length);
        }
        return wordToHexValue;
    };

    var x = [],
        k,
        AA,
        BB,
        CC,
        DD,
        a,
        b,
        c,
        d,
        S11 = 7,
        S12 = 12,
        S13 = 17,
        S14 = 22,
        S21 = 5,
        S22 = 9,
        S23 = 14,
        S24 = 20,
        S31 = 4,
        S32 = 11,
        S33 = 16,
        S34 = 23,
        S41 = 6,
        S42 = 10,
        S43 = 15,
        S44 = 21;

    str = this.utf8_encode(str);
    x = convertToWordArray(str);
    a = 0x67452301;
    b = 0xEFCDAB89;
    c = 0x98BADCFE;
    d = 0x10325476;

    xl = x.length;
    for ( k = 0; k < xl; k += 16) {
        AA = a;
        BB = b;
        CC = c;
        DD = d;
        a = _FF(a, b, c, d, x[k + 0], S11, 0xD76AA478);
        d = _FF(d, a, b, c, x[k + 1], S12, 0xE8C7B756);
        c = _FF(c, d, a, b, x[k + 2], S13, 0x242070DB);
        b = _FF(b, c, d, a, x[k + 3], S14, 0xC1BDCEEE);
        a = _FF(a, b, c, d, x[k + 4], S11, 0xF57C0FAF);
        d = _FF(d, a, b, c, x[k + 5], S12, 0x4787C62A);
        c = _FF(c, d, a, b, x[k + 6], S13, 0xA8304613);
        b = _FF(b, c, d, a, x[k + 7], S14, 0xFD469501);
        a = _FF(a, b, c, d, x[k + 8], S11, 0x698098D8);
        d = _FF(d, a, b, c, x[k + 9], S12, 0x8B44F7AF);
        c = _FF(c, d, a, b, x[k + 10], S13, 0xFFFF5BB1);
        b = _FF(b, c, d, a, x[k + 11], S14, 0x895CD7BE);
        a = _FF(a, b, c, d, x[k + 12], S11, 0x6B901122);
        d = _FF(d, a, b, c, x[k + 13], S12, 0xFD987193);
        c = _FF(c, d, a, b, x[k + 14], S13, 0xA679438E);
        b = _FF(b, c, d, a, x[k + 15], S14, 0x49B40821);
        a = _GG(a, b, c, d, x[k + 1], S21, 0xF61E2562);
        d = _GG(d, a, b, c, x[k + 6], S22, 0xC040B340);
        c = _GG(c, d, a, b, x[k + 11], S23, 0x265E5A51);
        b = _GG(b, c, d, a, x[k + 0], S24, 0xE9B6C7AA);
        a = _GG(a, b, c, d, x[k + 5], S21, 0xD62F105D);
        d = _GG(d, a, b, c, x[k + 10], S22, 0x2441453);
        c = _GG(c, d, a, b, x[k + 15], S23, 0xD8A1E681);
        b = _GG(b, c, d, a, x[k + 4], S24, 0xE7D3FBC8);
        a = _GG(a, b, c, d, x[k + 9], S21, 0x21E1CDE6);
        d = _GG(d, a, b, c, x[k + 14], S22, 0xC33707D6);
        c = _GG(c, d, a, b, x[k + 3], S23, 0xF4D50D87);
        b = _GG(b, c, d, a, x[k + 8], S24, 0x455A14ED);
        a = _GG(a, b, c, d, x[k + 13], S21, 0xA9E3E905);
        d = _GG(d, a, b, c, x[k + 2], S22, 0xFCEFA3F8);
        c = _GG(c, d, a, b, x[k + 7], S23, 0x676F02D9);
        b = _GG(b, c, d, a, x[k + 12], S24, 0x8D2A4C8A);
        a = _HH(a, b, c, d, x[k + 5], S31, 0xFFFA3942);
        d = _HH(d, a, b, c, x[k + 8], S32, 0x8771F681);
        c = _HH(c, d, a, b, x[k + 11], S33, 0x6D9D6122);
        b = _HH(b, c, d, a, x[k + 14], S34, 0xFDE5380C);
        a = _HH(a, b, c, d, x[k + 1], S31, 0xA4BEEA44);
        d = _HH(d, a, b, c, x[k + 4], S32, 0x4BDECFA9);
        c = _HH(c, d, a, b, x[k + 7], S33, 0xF6BB4B60);
        b = _HH(b, c, d, a, x[k + 10], S34, 0xBEBFBC70);
        a = _HH(a, b, c, d, x[k + 13], S31, 0x289B7EC6);
        d = _HH(d, a, b, c, x[k + 0], S32, 0xEAA127FA);
        c = _HH(c, d, a, b, x[k + 3], S33, 0xD4EF3085);
        b = _HH(b, c, d, a, x[k + 6], S34, 0x4881D05);
        a = _HH(a, b, c, d, x[k + 9], S31, 0xD9D4D039);
        d = _HH(d, a, b, c, x[k + 12], S32, 0xE6DB99E5);
        c = _HH(c, d, a, b, x[k + 15], S33, 0x1FA27CF8);
        b = _HH(b, c, d, a, x[k + 2], S34, 0xC4AC5665);
        a = _II(a, b, c, d, x[k + 0], S41, 0xF4292244);
        d = _II(d, a, b, c, x[k + 7], S42, 0x432AFF97);
        c = _II(c, d, a, b, x[k + 14], S43, 0xAB9423A7);
        b = _II(b, c, d, a, x[k + 5], S44, 0xFC93A039);
        a = _II(a, b, c, d, x[k + 12], S41, 0x655B59C3);
        d = _II(d, a, b, c, x[k + 3], S42, 0x8F0CCC92);
        c = _II(c, d, a, b, x[k + 10], S43, 0xFFEFF47D);
        b = _II(b, c, d, a, x[k + 1], S44, 0x85845DD1);
        a = _II(a, b, c, d, x[k + 8], S41, 0x6FA87E4F);
        d = _II(d, a, b, c, x[k + 15], S42, 0xFE2CE6E0);
        c = _II(c, d, a, b, x[k + 6], S43, 0xA3014314);
        b = _II(b, c, d, a, x[k + 13], S44, 0x4E0811A1);
        a = _II(a, b, c, d, x[k + 4], S41, 0xF7537E82);
        d = _II(d, a, b, c, x[k + 11], S42, 0xBD3AF235);
        c = _II(c, d, a, b, x[k + 2], S43, 0x2AD7D2BB);
        b = _II(b, c, d, a, x[k + 9], S44, 0xEB86D391);
        a = addUnsigned(a, AA);
        b = addUnsigned(b, BB);
        c = addUnsigned(c, CC);
        d = addUnsigned(d, DD);
    }

    var temp = wordToHex(a) + wordToHex(b) + wordToHex(c) + wordToHex(d);

    return temp.toLowerCase();
}

/***
 * @function utf8_encode
 * @description utf8编码
 * @param {string} argString 需要编码的字符串
 * @returns {string} utftext 编码后的字符串
 */
function utf8_encode(argString) {
    if (argString === null || typeof argString === 'undefined') {
        return '';
    }

    var string = (argString + '');
    // .replace(/\r\n/g, "\n").replace(/\r/g, "\n");
    var utftext = '',
        start,
        end,
        stringl = 0;

    start = end = 0;
    stringl = string.length;
    for (var n = 0; n < stringl; n++) {
        var c1 = string.charCodeAt(n);
        var enc = null;

        if (c1 < 128) {
            end++;
        } else if (c1 > 127 && c1 < 2048) {
            enc = String.fromCharCode((c1 >> 6) | 192, (c1 & 63) | 128);
        } else if ((c1 & 0xF800) != 0xD800) {
            enc = String.fromCharCode((c1 >> 12) | 224, ((c1 >> 6) & 63) | 128, (c1 & 63) | 128);
        } else {// surrogate pairs
            if ((c1 & 0xFC00) != 0xD800) {
                throw new RangeError('Unmatched trail surrogate at ' + n);
            }
            var c2 = string.charCodeAt(++n);
            if ((c2 & 0xFC00) != 0xDC00) {
                throw new RangeError('Unmatched lead surrogate at ' + (n - 1));
            }
            c1 = ((c1 & 0x3FF) << 10) + (c2 & 0x3FF) + 0x10000;
            enc = String.fromCharCode((c1 >> 18) | 240, ((c1 >> 12) & 63) | 128, ((c1 >> 6) & 63) | 128, (c1 & 63) | 128);
        }
        if (enc !== null) {
            if (end > start) {
                utftext += string.slice(start, end);
            }
            utftext += enc;
            start = end = n + 1;
        }
    }

    if (end > start) {
        utftext += string.slice(start, stringl);
    }

    return utftext;
}

/**
 * @function upload
 * @description 上传文件
 * @param {string} id 唯一标识
 * @param {string} success 成功回调函数
 *                         函数参数为serverPath，即服务器端返回的数据
 * @param {string|function} error 错误回调函数或提示
 *                                函数参数为opCode, fileSize, percent, serverPath, status
 * @param {string|function} uploading 上传中回调函数或提示
 *                                    函数参数为opCode, fileSize, percent, serverPath, status
 * @param {string} file 文件路径
 * @param {string} file_name 服务器端接收时用到的文件名称
 * @param {string} url 唯一标识
 */
function upload(id, success, error, uploading, file, file_name, url) {
    if ( typeof id == 'object' && arguments.length == 1) {
        success = id.success
        error = id.error
        uploading = id.uploading
        file = id.file
        file_name = id.file_name
        url = id.url
        id = id.id
    }

    uexUploaderMgr.onStatus = function(opCode, fileSize, percent, serverPath, status) {
        switch (status) {
        case 0:
            if ( typeof uploading == 'function') {
                uploading(opCode, fileSize, percent, serverPath, status)
            } else if ( typeof uploading == 'string') {
                open_toast(uploading, -1)
            } else {
                open_toast('上传中', -1)
            }
            break
        case 1:
            uexUploaderMgr.closeUploader(opCode)
            close_toast()
            if ( typeof success == 'function') {
                success(serverPath)
            }
            break
        case 2:
            uexUploaderMgr.closeUploader(opCode)
            close_toast()
            if ( typeof error == 'function') {
                error(opCode, fileSize, percent, serverPath, status)
            } else if ( typeof error == 'string') {
                open_toast(error)
            } else {
                open_toast('上传失败')
            }
            break
        }
    }

    uexUploaderMgr.cbCreateUploader = function(opCode, dataType, data) {
        if (data == 0) {
            var header = {
                'APPVERIFY' : get_verify()
            }
            uexUploaderMgr.setHeaders(opCode, JSON.stringify(header))
            uexUploaderMgr.uploadFile(opCode, file, file_name)
        } else {
            if ( typeof error == 'function') {
                error()
            } else if ( typeof error == 'string') {
                open_toast(error)
            } else {
                open_toast('上传失败')
            }
        }
    }
    uexUploaderMgr.createUploader(id, url)
}

/**
 * @function open_area
 * @description 打开选择城市窗口
 */
function open_area() {
    open_page('area.html', 'area', '选择城市')
}

/**
 * @function ksort
 * @decription 按键进行排序
 * @param {array|object} inputArr 需要进行排序的数组或对象
 * @param {string} sort_flags 排序方式
 * @returns {array|object} 排序后的数组或对象
 */
function ksort(inputArr, sort_flags) {
    var tmp_arr = {},
        keys = [],
        sorter,
        i,
        k,
        that = this,
        strictForIn = false,
        populateArr = {};

    switch (sort_flags) {
    case 'SORT_STRING':
        // compare items as strings
        sorter = function(a, b) {
            return that.strnatcmp(a, b);
        };
        break;
    case 'SORT_LOCALE_STRING':
        // compare items as strings, original by the current locale (set with  i18n_loc_set_default() as of PHP6)
        var loc = this.i18n_loc_get_default();
        sorter = this.php_js.i18nLocales[loc].sorting;
        break;
    case 'SORT_NUMERIC':
        // compare items numerically
        sorter = function(a, b) {
            return ((a + 0) - (b + 0));
        };
        break;
    // case 'SORT_REGULAR': // compare items normally (don't change types)
    default:
        sorter = function(a, b) {
            var aFloat = parseFloat(a),
                bFloat = parseFloat(b),
                aNumeric = aFloat + '' === a,
                bNumeric = bFloat + '' === b;
            if (aNumeric && bNumeric) {
                return aFloat > bFloat ? 1 : aFloat < bFloat ? -1 : 0;
            } else if (aNumeric && !bNumeric) {
                return 1;
            } else if (!aNumeric && bNumeric) {
                return -1;
            }
            return a > b ? 1 : a < b ? -1 : 0;
        };
        break;
    }

    // Make a list of key names
    for (k in inputArr) {
        if (inputArr.hasOwnProperty(k)) {
            keys.push(k);
        }
    }
    keys.sort(sorter);

    // BEGIN REDUNDANT
    this.php_js = this.php_js || {};
    this.php_js.ini = this.php_js.ini || {};
    // END REDUNDANT
    strictForIn = this.php_js.ini['phpjs.strictForIn'] && this.php_js.ini['phpjs.strictForIn'].local_value && this.php_js
    .ini['phpjs.strictForIn'].local_value !== 'off';
    populateArr = strictForIn ? inputArr : populateArr;

    // Rebuild array with sorted key names
    for ( i = 0; i < keys.length; i++) {
        k = keys[i];
        tmp_arr[k] = inputArr[k];
        if (strictForIn) {
            delete inputArr[k];
        }
    }
    for (i in tmp_arr) {
        if (tmp_arr.hasOwnProperty(i)) {
            populateArr[i] = tmp_arr[i];
        }
    }

    return strictForIn || populateArr;
}

/**
 * @function weixin_sign
 * @description 微信加密
 *              将参数按照键进行排序
 *              然后key=val&的格式拼接字符串
 *              然后在最后加入key=WEIXIN_KEY
 *              其中WEIXIN_KEY在config.js有详细说明
 *              最后使用md5函数加密
 * @returns {string} md5 加密后的字符串
 */
function weixin_sign(parameters) {
    var str = ''
    for (x in parameters) {
        str += x + '=' + parameters[x] + '&'
    }

    str += 'key=' + WEIXIN_KEY
    return md5(str).toUpperCase()
}

/**
 * @function
 * @name get_checkbox_value
 * @description 获取指定的选择框的值
 * @param {string} id 选择框唯一标识
 * @param {string} name 选择框名称
 * @returns {array} 包含所有选定状态的选择框的值的数组
 */
function get_checkbox_value(id, name) {
    var result = get_checkbox_target.apply(this, arguments)
    if (!result) {
        return false
    }
    var tmp = new Array()
    for (i in result) {
        tmp[i] = $(result[i]).attr('value')
    }
    return tmp
}

/**
 * @function
 * @name get_checkbox_value
 * @description 获取指定的选择框的文本
 * @param {string} id 选择框唯一标识
 * @param {string} name 选择框名称
 * @returns {array} 包含所有选定状态的选择框的文本的数组
 */
function get_checkbox_html(id, name) {
    var result = get_checkbox_target.apply(this, arguments)
    if (!result) {
        return false
    }
    var tmp = new Array()
    for (i in result) {
        tmp[i] = $(result[i]).html()
    }
    return tmp
}

/**
 * @function
 * @name get_checkbox_target
 * @description 获取符合条件的选择框元素
 * @param {string} id 选择框唯一标识
 * @param {string} name 选择框名称
 * @param {string} value 选择框的值
 * @param {boolean} checked 是否为选定状态，默认为true
 *                          如果填false，则忽略选定状态
 *
 * @returns {array} 包含所有选定状态的选择框的数组
 */
function get_checkbox_target(id, name, value, checked) {
    if ($.isPlainObject(id) && arguments.length == 1) {
        name = id.name
        value = id.value
        checked = id.checked
        id = id.id
    }
    if ( typeof id == 'undefined' && typeof name == 'undefined') {
        return false
    }
    if ( typeof id != 'undefined') {
        return $('#' + id)
    }
    if ( typeof checked == 'undefined') {
        checked = true
    }
    if ( typeof name != 'undefined') {
        var result = new Array()
        var exist = false
        name = name.split('|')
        $('._checkbox').each(function(index) {
            var cur_name = $(this).attr('name')
            if ( typeof cur_name != 'undefined' && cur_name != null) {
                cur_name = $(this).attr('name').split('|')
                var cur_value = $(this).attr('value')
                var cur_checked = $(this).prop('checked')
                if (_.union(name, cur_name).length == cur_name.length) {
                    exist = true
                    if (( typeof value === 'undefined' || value == cur_value) && (checked == false || cur_checked == true)) {
                        result.push($(this))
                    }
                }
            }
        })
        if (exist === false) {
            return false
        } else {
            return result
        }
    } else {
        return false
    }
}

/**
 * @function page_channel_func
 * @description 消息入口
 * @param {string} msg 消息内容
 */
function page_channel_func(msg) {
    try {
        msg = JSON.parse(msg)
    } catch(e) {
        console.log('JSON Parse error:' + e + ',content:' + msg)
    }
    var channel_id = msg['channel_id']
    var msg_page_name = msg['page_name']
    var msg_page_type = msg['page_type']
    delete msg['page_name']
    delete msg['page_type']
    delete msg['channel_id']
    if ( typeof msg['_msg'] != 'undefined') {
        var msg = msg['_msg']
    } else if (!_.isEmpty(msg)) {
        var tmp = new Object()
        for (i in msg) {
            tmp[i] = msg[i]
        }
    }
    for (var i in page_channel_func_arr[channel_id]) {
        if ( typeof page_channel_func_arr[channel_id][i] === 'function') {
            page_channel_func_arr[channel_id][i](msg, msg_page_name, msg_page_type)
        }
    }
}

/**
 * @function open_chat
 * @description 打开即时通讯窗口
 * @param {string} supplier_id 店铺ID
 * @param {string} goods_id 商品ID
 * @param {string} order_id 订单ID
 * @param {string} order_sb 订单号
 */
function open_chat(supplier_id, goods_id, order_id, order_sn) {
    if ($.isPlainObject(supplier_id) && arguments.length == 1) {
        goods_id = supplier_id.goods_id
        order_id = supplier_id.order_id
        order_sn = supplier_id.order_sn
        supplier_id = supplier_id.supplier_id
    }
    if ( typeof supplier_id == 'undefined') {
        supplier_id = ''
    }
    if ( typeof goods_id == 'undefined') {
        goods_id = ''
    }
    if ( typeof order_id == 'undefined') {
        order_id = ''
    }
    if ( typeof order_sn == 'undefined') {
        order_sn = ''
    }
    set_val('supplier_id', supplier_id)
    set_val('goods_id', goods_id)
    set_val('order_id', order_id)
    set_val('order_sn', order_sn)
    var win_name = 'chat_' + supplier_id + '_' + goods_id + '_' + order_id + '_' + order_sn
    open_page('chat.html', win_name, '在线客服')
}

/**
 * @function open_brand
 * @param {string} brand_id 品牌ID
 * @description 打开指定品牌商品列表
 */
function open_brand(brand_id) {
    open_goods_list({
        brand_id : brand_id
    })
}
/**
 * @function open_brands
 * @param {string} brand_id 品牌ID
 * @description 打开指定品牌信息
 */
function open_brands(brand_id,brand_name) {    
    set_val('brand_id', brand_id)
    open_page('brand.html', 'brand_' + brand_id,brand_name)
}

/**
 * @function init_end_time
 * @description 初始化倒计时
 * @param {number} now 当前时间戳 单位：秒
 */
function init_end_time(now) {
    var time = now || global.now_time || (_.now() / 1000)

    $(".settime").each(function(i) {
        var endTime = $(this).attr('endTime')
        var lag = endTime - time
        if (lag > 0) {
            var second = Math.floor(lag % 60)
            var minite = Math.floor((lag / 60) % 60)
            var hour = Math.floor((lag / 3600) % 24)
            var day = Math.floor((lag / 3600) / 24)
            $(this).html(day + '天' + hour + '小时' + minite + '分' + second + '秒')
        } else {
            $(this).html('活动已经结束啦！')
        }
    })
    global.end_time = setTimeout('init_end_time(' + (time - 0 + 1) + ')', DUR_SECOND)
}

/**
 * @function init_switch
 * @description 初始化switch
 */
function init_switch() {
    $('._switch').each(function() {
        var checked = $(this).data('checked') || false
        if (checked === false) {
            $(this).removeClass('bc-head')
        } else {
            $(this).addClass('bc-head')
        }
        $(this).on('click', function() {
            var disabled = $(this).data('disabled') || false
            if (!disabled) {
                var checked = $(this).data('checked') || false
                checked = !checked
                $(this).data("checked", checked)
                if (checked === false) {
                    $(this).removeClass('bc-head')
                } else {
                    $(this).addClass('bc-head')
                }
                $(this).trigger('change', [this, checked])
            }
        })
    })
}

/**
 * @function unique_page_id
 * @description 根据page_name和page_type生成唯一标识
 * @param {string} name page_name 窗口名称 ，默认为page_name
 * @param {string} type page_type 窗口类型，默认为page_type
 * @return {string} unique_id 唯一标识
 */
function unique_page_id(name, type) {
    if ( typeof name === 'undefined') {
        name = page_name
    }
    if ( typeof type === 'undefined') {
        type = page_type
    }

    return page_name + '_' + page_type
}
